-------------------------------------------------------------------------------
-- Title      : WR Switch 8-ports top level
-- Project    : White Rabbit Switch
-------------------------------------------------------------------------------
-- File       : scb_top_synthesis.vhd
-- Author     : Tomasz Wlostowski, Maciej Lipinski, Grzegorz Daniluk
-- Company    : CERN BE-CO-HT
-- Created    : 2012-03-07
-- Last update: 2020-08-18
-- Platform   : FPGA-generic
-- Standard   : VHDL
-------------------------------------------------------------------------------
-- Description:
-- WR Switch 8-port version top synthesis level.
-------------------------------------------------------------------------------
--
-- Copyright (c) 2012 - 2014 CERN / BE-CO-HT
--
-- This source file is free software; you can redistribute it
-- and/or modify it under the terms of the GNU Lesser General
-- Public License as published by the Free Software Foundation;
-- either version 2.1 of the License, or (at your option) any
-- later version.
--
-- This source is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied
-- warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-- PURPOSE.  See the GNU Lesser General Public License for more
-- details.
--
-- You should have received a copy of the GNU Lesser General
-- Public License along with this source; if not, download it
-- from http://www.gnu.org/licenses/lgpl-2.1.html
--
-------------------------------------------------------------------------------

library ieee;
use ieee.STD_LOGIC_1164.all;
use ieee.numeric_std.all;

library work;
use work.wishbone_pkg.all;
use work.gencores_pkg.all;
use work.wr_fabric_pkg.all;
use work.endpoint_pkg.all;
use work.wr_txtsu_pkg.all;
use work.wrsw_top_pkg.all;
use work.wrsw_shared_types_pkg.all;
use work.ps_wrapper;

library UNISIM;
use UNISIM.vcomponents.all;


entity afcz_wrs_8p_top is
  generic(
    g_simulation     : boolean := false;
    g_with_ibert     : boolean := false
    );
  port (
    

    -- DMTD clock
    --clk_20m_vcxo_i : in std_logic;        

    --FMC(1) clocks
    clk_aux_p_i     : in std_logic;
    clk_aux_n_i     : in std_logic;
    clk_helper_p_i  : in std_logic;
    clk_helper_n_i  : in std_logic;
    clk_ref_p_i     : in std_logic;
    clk_ref_n_i     : in std_logic;

    i2c_scl_b : inout std_logic; -- Global I2C Bus (U30/U31 mux)
    i2c_sda_b : inout std_logic;

    si57x_scl_b : inout std_logic; -- Si570 dedicated I2C pins (U5/RT subsytem
                                   -- only)
    si57x_sda_b : inout std_logic;
    
    --sfp i2c busses
    sfp_scl_b  : inout std_logic_vector(3 downto 0);
    sfp_sda_b  : inout std_logic_vector(3 downto 0);

    -- DAC Control
    wr_dac1_sync_n_o : out std_logic_vector(1 downto 0);
    wr_dac1_sclk_o   : out std_logic;
    wr_dac1_din_o    : out std_logic;
    
    uart_txd_o : out std_logic;
    uart_rxd_i : in  std_logic;

    fmc_3v8_enable_o         : out std_logic;
    hmc_sdata_b     : inout std_logic;
    hmc_sclk_o      : out std_logic;
    hmc_slen_n_o    : out std_logic;
    hmc_sync_p_o    : out std_logic;
    hmc_sync_n_o    : out std_logic;
    hmc_reset_o     : out std_logic;

    -- GTHE4 ports
    mgtclk1_224_p_i : in std_logic;
    mgtclk1_224_n_i : in std_logic;

    mgtclk0_227_p_i : in std_logic;
    mgtclk0_227_n_i : in std_logic;
    mgtclk1_227_p_i : in std_logic;
    mgtclk1_227_n_i : in std_logic;

    mgtclk0_226_p_i : in std_logic;
    mgtclk0_226_n_i : in std_logic;
    mgtclk1_226_p_i : in std_logic;
    mgtclk1_226_n_i : in std_logic;

    mgtclk1_225_p_i : in std_logic;
    mgtclk1_225_n_i : in std_logic;    

    mgt227_tx_p_o : out std_logic_vector(3 downto 0);
    mgt227_tx_n_o : out std_logic_vector(3 downto 0);
    mgt227_rx_p_i : in  std_logic_vector(3 downto 0);
    mgt227_rx_n_i : in  std_logic_vector(3 downto 0)

    --mgt225_tx_p_o : out std_logic_vector(3 downto 0);
    --mgt225_tx_n_o : out std_logic_vector(3 downto 0);
    --mgt225_rx_p_i : in  std_logic_vector(3 downto 0);
    --mgt225_rx_n_i : in  std_logic_vector(3 downto 0)  
  );

end afcz_wrs_8p_top;

architecture Behavioral of afcz_wrs_8p_top is
  
  constant c_NUM_PHYS  : integer := 4;
  constant c_NUM_PORTS : integer := 4;

  function f_bool2int(x : boolean) return integer is
  begin
    if(x) then
      return 1;
    else
      return 0;
    end if;
  end f_bool2int;


  signal to_phys   : t_phyif_output_array(c_NUM_PHYS-1 downto 0);
  signal from_phys : t_phyif_input_array(c_NUM_PHYS-1 downto 0);

  signal clk_aux      : std_logic;
  signal clk_gtx      : std_logic_vector(c_NUM_PHYS-1 downto 0);

  signal axi4l_aclk     : std_logic;
  signal axi4l_aresetn  : std_logic;
  signal  axi4l_arvalid :  std_logic;
  signal  axi4l_awvalid :  std_logic;
  signal  axi4l_bready  :  std_logic;
  signal  axi4l_rready  :  std_logic;
  signal  axi4l_wlast   :  std_logic := '1';
  signal  axi4l_wvalid  :  std_logic;
  signal  axi4l_araddr  :  std_logic_vector (31 downto 0);
  signal  axi4l_awaddr  :  std_logic_vector (31 downto 0);
  signal  axi4l_wdata   :  std_logic_vector (31 downto 0);
  signal  axi4l_wstrb   :  std_logic_vector (3 downto 0);
  signal  axi4l_arready :  std_logic;
  signal  axi4l_awready :  std_logic;
  signal  axi4l_bvalid  :  std_logic;
  signal  axi4l_rlast   :  std_logic;
  signal  axi4l_rvalid  :  std_logic;
  signal  axi4l_wready  :  std_logic;
  signal  axi4l_bresp   :  std_logic_vector (1 downto 0);
  signal  axi4l_rresp   :  std_logic_vector (1 downto 0);
  signal  axi4l_rdata   :  std_logic_vector (31 downto 0);  

  signal sys_rst_n_i    : std_logic;         -- global reset
  signal clk_startup_i  : std_logic; -- startup clock, 100 MHz (PCIe core)  
  
  component scb_top_bare
    generic (
      g_num_ports       : integer;
      g_simulation      : boolean;
      g_without_network : boolean;
      g_with_TRU        : boolean;
      g_with_TATSU      : boolean;
      g_with_HWIU       : boolean;
      g_with_PSTATS     : boolean;
      g_with_muxed_CS   : boolean;
      g_with_watchdog   : boolean;
      g_with_rtu        : boolean;       
      g_with_swc        : boolean;
      g_with_si57x      : boolean;
      g_si57x_is_helper : boolean;
      g_inj_per_EP      : std_logic_vector(17 downto 0);
      g_phy_lpcalib     : t_bool_array(0 to 17) := c_BOOL_FALSE_ARRAY;
      g_rtcpu_swap_host_endian : boolean
    );
    port (
      sys_rst_n_i         : in  std_logic;
      clk_startup_i       : in  std_logic;
      clk_ref_i           : in  std_logic;
      clk_dmtd_i          : in  std_logic;
      clk_aux_i           : in  std_logic;
      clk_ext_mul_i       : in  std_logic_vector(1 downto 0);
      clk_ext_mul_locked_i: in  std_logic;
      clk_aux_p_o         : out std_logic;
      clk_aux_n_o         : out std_logic;
      clk_500_o           : out std_logic;
      clk_sys_o           : out std_logic;
      cpu_wb_i            : in  t_wishbone_slave_in;
      cpu_wb_o            : out t_wishbone_slave_out;
      cpu_irq_n_o         : out std_logic;
      pps_i               : in  std_logic;
      ppsin_term_o        : out std_logic;
      pps_o               : out std_logic;
      ljd_dac_main_sync_n_o : out std_logic;
      ljd_dac_main_sclk_o   : out std_logic;
      ljd_dac_main_data_o   : out std_logic;
      ljd_loopback_i        : in  std_logic;
      ljd_loopback_o        : out std_logic;
      ljd_clk1_en           : out std_logic;
      ljd_clk2_en           : out std_logic;
      ljd_detected_o        : out std_logic;
      ljd_osc_freq_i        : in  std_logic_vector (2 downto 0);
      ljd_pll_mosi_o        : out std_logic;
      ljd_pll_miso_i        : in  std_logic;
      ljd_pll_sck_o         : out std_logic;
      ljd_pll_cs_n_o        : out std_logic;
      ljd_pll_sync_n_o      : out std_logic;
      ljd_pll_reset_n_o     : out std_logic;
      ljd_pll_locked_i      : in  std_logic;
      pll_status_i        : in  std_logic;
      pll_mosi_o          : out std_logic;
      pll_miso_i          : in  std_logic;
      pll_sck_o           : out std_logic;
      pll_cs_n_o          : out std_logic;
      pll_sync_n_o        : out std_logic;
      pll_reset_n_o       : out std_logic;
      uart_txd_o          : out std_logic;
      uart_rxd_i          : in  std_logic;
      clk_en_o            : out std_logic;
      clk_sel_o           : out std_logic;
      uart_sel_o          : out std_logic;
      clk_dmtd_divsel_o   : out std_logic;
      phys_o              : out t_phyif_output_array(g_num_ports-1 downto 0);
      phys_i              : in  t_phyif_input_array(g_num_ports-1 downto 0);
      led_link_o          : out std_logic_vector(g_num_ports-1 downto 0);
      led_act_o           : out std_logic_vector(g_num_ports-1 downto 0);
      gpio_o              : out std_logic_vector(31 downto 0);
      gpio_i              : in  std_logic_vector(31 downto 0);
      i2c_scl_oen_o   : out std_logic_vector(2 downto 0);
      i2c_scl_o       : out std_logic_vector(2 downto 0);
      i2c_scl_i       : in  std_logic_vector(2 downto 0) := "111";
      i2c_sda_oen_o   : out std_logic_vector(2 downto 0);
      i2c_sda_o       : out std_logic_vector(2 downto 0);
      i2c_sda_i       : in  std_logic_vector(2 downto 0) := "111";
      mb_fan1_pwm_o : out std_logic;
      mb_fan2_pwm_o : out std_logic;
      spll_dbg_o    : out std_logic_vector(5 downto 0);

      rts_si57x_scl_oen_o : out std_logic;
      rts_si57x_sda_oen_o : out std_logic;
      rts_si57x_scl_i : in std_logic;
      rts_si57x_sda_i : in std_logic;

      --sfp i2c ports
      sfp_i2c_scl_oen_o   : out std_logic_vector(g_num_ports-1 downto 0);
      sfp_i2c_scl_o       : out std_logic_vector(g_num_ports-1 downto 0);
      sfp_i2c_scl_i       : in std_logic_vector(g_num_ports-1 downto 0);
      sfp_i2c_sda_oen_o   : out std_logic_vector(g_num_ports-1 downto 0);
      sfp_sda_o           : out std_logic_vector(g_num_ports-1 downto 0);
      sfp_sda_i           : out std_logic_vector(g_num_ports-1 downto 0);
      sfp_tx_disable_o    : out std_logic_vector(g_num_ports-1 downto 0);

      dbg_gpio_o          : out std_logic_vector(7 downto 0);
      dbg_gpio_i          : in std_logic_vector(7 downto 0)
      );
  end component;

  constant c_NUM_GTH_QUADS    : integer := 1;
  constant c_NUM_GTH_REFCLKS  : integer := 1;

  component ibert_ultrascale_gth_0
    port (
      txn_o : OUT STD_LOGIC_VECTOR((4*c_NUM_GTH_QUADS)-1 DOWNTO 0);
      txp_o : OUT STD_LOGIC_VECTOR((4*c_NUM_GTH_QUADS)-1 DOWNTO 0);
      rxoutclk_o : OUT STD_LOGIC_VECTOR((4*c_NUM_GTH_QUADS)-1 DOWNTO 0);
      rxn_i : IN STD_LOGIC_VECTOR((4*c_NUM_GTH_QUADS)-1 DOWNTO 0);
      rxp_i : IN STD_LOGIC_VECTOR((4*c_NUM_GTH_QUADS)-1 DOWNTO 0);
      gtrefclk0_i : IN STD_LOGIC_VECTOR(c_NUM_GTH_REFCLKS-1 DOWNTO 0);
      gtrefclk1_i : IN STD_LOGIC_VECTOR(c_NUM_GTH_REFCLKS-1 DOWNTO 0);
      gtnorthrefclk0_i : IN STD_LOGIC_VECTOR(c_NUM_GTH_REFCLKS-1 DOWNTO 0);
      gtnorthrefclk1_i : IN STD_LOGIC_VECTOR(c_NUM_GTH_REFCLKS-1 DOWNTO 0);
      gtsouthrefclk0_i : IN STD_LOGIC_VECTOR(c_NUM_GTH_REFCLKS-1 DOWNTO 0);
      gtsouthrefclk1_i : IN STD_LOGIC_VECTOR(c_NUM_GTH_REFCLKS-1 DOWNTO 0);
      gtrefclk00_i : IN STD_LOGIC_VECTOR(c_NUM_GTH_REFCLKS-1 DOWNTO 0);
      gtrefclk10_i : IN STD_LOGIC_VECTOR(c_NUM_GTH_REFCLKS-1 DOWNTO 0);
      gtrefclk01_i : IN STD_LOGIC_VECTOR(c_NUM_GTH_REFCLKS-1 DOWNTO 0);
      gtrefclk11_i : IN STD_LOGIC_VECTOR(c_NUM_GTH_REFCLKS-1 DOWNTO 0);
      gtnorthrefclk00_i : IN STD_LOGIC_VECTOR(c_NUM_GTH_REFCLKS-1 DOWNTO 0);
      gtnorthrefclk10_i : IN STD_LOGIC_VECTOR(c_NUM_GTH_REFCLKS-1 DOWNTO 0);
      gtnorthrefclk01_i : IN STD_LOGIC_VECTOR(c_NUM_GTH_REFCLKS-1 DOWNTO 0);
      gtnorthrefclk11_i : IN STD_LOGIC_VECTOR(c_NUM_GTH_REFCLKS-1 DOWNTO 0);
      gtsouthrefclk00_i : IN STD_LOGIC_VECTOR(c_NUM_GTH_REFCLKS-1 DOWNTO 0);
      gtsouthrefclk10_i : IN STD_LOGIC_VECTOR(c_NUM_GTH_REFCLKS-1 DOWNTO 0);
      gtsouthrefclk01_i : IN STD_LOGIC_VECTOR(c_NUM_GTH_REFCLKS-1 DOWNTO 0);
      gtsouthrefclk11_i : IN STD_LOGIC_VECTOR(c_NUM_GTH_REFCLKS-1 DOWNTO 0);
      clk : IN STD_LOGIC 
    );
  end component;

  signal clk_20m_vcxo_buf : std_logic;
  signal clk_dmtd_bufin : std_logic;
  signal clk_dmtd : std_logic;
  signal clk_dmtd_fbout : std_logic;
  signal clk_dmtd_fbin : std_logic;
  signal dmtd_pll_locked : std_logic;

  signal clk_helper_buf : std_logic;
  signal clk_aux_buf    : std_logic;

  signal rst_sys : std_logic;
  signal clk_sys : std_logic;

  signal sel_clk_sys : std_logic;
  
  signal clk_swcore_fbout, clk_swcore_fbin, clk_swcore_bufin : std_logic;
  signal swcore_pll_locked, swcore_pll_rst : std_logic;

  signal host_master_out : t_wishbone_master_out;
  signal host_master_in : t_wishbone_master_in;

  attribute mark_debug : string;
  attribute mark_debug of host_master_in :signal is "true";
  attribute mark_debug of host_master_out :signal is "true";
  attribute mark_debug of sel_clk_sys : signal is "true";
  
  signal gth_refclk : std_logic_vector(c_NUM_PHYS-1 downto 0);
  signal gth_txoutclk : std_logic_vector(c_NUM_PHYS-1 downto 0);
  signal clk_ref : std_logic;

  signal cpu_irq_n : std_logic;

  signal i2c_scl_oen : std_logic_vector(2 downto 0);
  signal i2c_sda_oen : std_logic_vector(2 downto 0);
  signal i2c_scl_out : std_logic_vector(2 downto 0);
  signal i2c_sda_out : std_logic_vector(2 downto 0);
  signal i2c_scl_in : std_logic_vector(2 downto 0);
  signal i2c_sda_in : std_logic_vector(2 downto 0);

  signal rts_si57x_scl_oen : std_logic;
  signal rts_si57x_sda_oen : std_logic;
  signal rts_si57x_scl_in : std_logic;
  signal rts_si57x_sda_in : std_logic;
  signal rts_gpio_out : std_logic_vector(7 downto 0);
  signal rts_gpio_in : std_logic_vector(7 downto 0);
  
  signal gth_txp, gth_rxp, gth_txn, gth_rxn : std_logic_vector(c_NUM_PHYS-1 downto 0);

  signal clk_gth : std_logic_vector(c_NUM_PHYS-1 downto 0);

  signal dac_main_value, dac_helper_value : std_logic_vector(15 downto 0);
  signal dac_main_load, dac_helper_load : std_logic;

  signal mgtclk1_224, mgtclk1_224_prebuf : std_logic;
  signal mgtclk0_227, mgtclk1_227 : std_logic;
  signal mgtclk0_226, mgtclk1_226 : std_logic;  
  signal mgtclk0_227_gclk : std_logic;
  signal mgtclk0_227_prebuf : std_logic;
  signal mgtclk1_225        : std_logic;
  signal mgtclk1_225_prebuf : std_logic;
  signal mgtclk1_225_gclk   : std_logic;
  
  signal aux_scl_oen, aux_sda_oen : std_logic;
  signal afcz_scl_oen, afcz_sda_oen : std_logic;
  signal aux_scl_in   : std_logic;
  signal aux_sda_in   : std_logic;
  signal aux_scl_out  : std_logic;
  signal aux_sda_out  : std_logic;
  
  --c_NUM_PORTS > c_num_sfp_ports...
  constant c_num_sfp_ports  : integer := 4;

  signal sfp_scl_in     : std_logic_vector(c_NUM_PORTS-1 downto 0);
  signal sfp_scl_out    : std_logic_vector(c_NUM_PORTS-1 downto 0);
  signal sfp_scl_oen    : std_logic_vector(c_NUM_PORTS-1 downto 0);
  signal sfp_sda_in     : std_logic_vector(c_NUM_PORTS-1 downto 0);
  signal sfp_sda_out    : std_logic_vector(c_NUM_PORTS-1 downto 0);
  signal sfp_sda_oen    : std_logic_vector(c_NUM_PORTS-1 downto 0);

  signal hmc_mosi_o     : std_logic;
  signal hmc_miso_i     : std_logic;
  signal hmc_sync_o     : std_logic;
  signal hmc_reset_n_o  : std_logic;
  signal hmc_gpio0_i    : std_logic;

  signal sys_rst_n_vec_i : std_logic_vector(0 downto 0);    --vivado bd generates a reset vector even though is only 1 reset...
  signal ibert_clk0      : std_logic_vector(1 downto 0);
  signal ibert_clk1      : std_logic_vector(1 downto 0);
  signal mgtclk0_226_prebuf : std_logic;
  signal mgtclk0_226_gclk   : std_logic;

  signal debug_gpo    : std_logic_vector(7 downto 0);
  signal debug_gpi    : std_logic_vector(7 downto 0);
  
begin

  sys_rst_n_i   <= sys_rst_n_vec_i(0);
  rst_sys <= not sys_rst_n_i;

  U_clk_helper_buf: IBUFDS 
    generic map 
    (
      DIFF_TERM   => TRUE,
      IOSTANDARD  => "LVDS"
    )
    port map 
    (
      I   => clk_helper_p_i,
      IB  => clk_helper_n_i,
      O   => clk_helper_buf
    );

  U_clk_aux_buf: IBUFDS 
    generic map 
    (
      DIFF_TERM   => TRUE,
      IOSTANDARD  => "LVDS"      
    )
    port map 
    (
      I   => clk_aux_p_i,
      IB  => clk_aux_n_i,
      O   => clk_aux_buf 
    );

  U_clk_ref_buf: IBUFDS 
    generic map 
    (
      DIFF_TERM   => TRUE,
      IOSTANDARD  => "LVDS"
    )
    port map 
    (
      I   => clk_ref_p_i,
      IB  => clk_ref_n_i,
      O   => clk_ref
    );

  U_SWCore_PLL : MMCME3_ADV
    generic map (
      BANDWIDTH            => "OPTIMIZED",
      CLKOUT4_CASCADE      => "FALSE",
      COMPENSATION         => "AUTO",
      STARTUP_WAIT         => "FALSE",
      DIVCLK_DIVIDE        => 1,
      CLKFBOUT_MULT_F      => 15.000,
      CLKFBOUT_PHASE       => 0.000,
      CLKFBOUT_USE_FINE_PS => "FALSE",
      CLKOUT0_DIVIDE_F     => 5.000,
      CLKOUT0_PHASE        => 0.000,
      CLKOUT0_DUTY_CYCLE   => 0.500,
      CLKOUT0_USE_FINE_PS  => "FALSE",
      CLKIN1_PERIOD        => 16.0)
    port map (
      CLKFBOUT => clk_swcore_fbout,
      CLKOUT0  => clk_swcore_bufin,
      CLKFBIN  => clk_swcore_fbin,
      CLKIN1   => clk_sys,
      CLKIN2   => '0',
      CLKINSEL => '1',
      DADDR    => (others => '0'),
      DCLK     => '0',
      DEN      => '0',
      DI       => (others => '0'),
      DWE      => '0',
      CDDCREQ  => '0',
      PSCLK    => '0',
      PSEN     => '0',
      PSINCDEC => '0',
      LOCKED   => swcore_pll_locked,
      PWRDWN   => '0',
      RST      => swcore_pll_rst);

  swcore_pll_rst <= '0'; --rts_gpio_out(0);
  rts_gpio_in(0) <= swcore_pll_locked;

  U_SWCore_Clock_PLL_Buf : BUFG
    port map (
      I => clk_swcore_bufin,
      O => clk_aux
      );

  U_SWCore_Clock_PLL_FB_Buf : BUFG
    port map (
      I => clk_swcore_fbout,
      O => clk_swcore_fbin
      );

  --zynq processor system top level wrapper
  U_ps_core : entity work.ps_wrapper
  port map
  (
    m_axi_o_araddr   => axi4l_araddr,
    m_axi_o_arprot   => open,
    m_axi_o_arready  => axi4l_arready,
    m_axi_o_arvalid  => axi4l_arvalid,
    m_axi_o_awaddr   => axi4l_awaddr,
    m_axi_o_awprot   => open,
    m_axi_o_awready  => axi4l_awready,
    m_axi_o_awvalid  => axi4l_awvalid,
    m_axi_o_bready   => axi4l_bready,
    m_axi_o_bresp    => axi4l_bresp,
    m_axi_o_bvalid   => axi4l_bvalid,
    m_axi_o_rdata    => axi4l_rdata,
    m_axi_o_rready   => axi4l_rready,
    m_axi_o_rresp    => axi4l_rresp,
    m_axi_o_rvalid   => axi4l_rvalid,
    m_axi_o_wdata    => axi4l_wdata,
    m_axi_o_wready   => axi4l_wready,
    m_axi_o_wstrb    => axi4l_wstrb,
    m_axi_o_wvalid   => axi4l_wvalid,
    axi_clk_i        => clk_sys,
    cpu_irq_n_i      => cpu_irq_n, 
    pl_clk_o         => clk_startup_i,
    pl_rst_n_o       => sys_rst_n_vec_i
  );
  
  U_axi4lite_bridge : entity work.wb_axi4lite_bridge
    port map (
      clk_sys_i  => clk_sys, 
      rst_n_i    => sys_rst_n_i, 
      ARVALID    => axi4l_arvalid, 
      AWVALID    => axi4l_awvalid, 
      BREADY     => axi4l_bready, 
      RREADY     => axi4l_rready,
      WLAST      => axi4l_wlast,  
      WVALID     => axi4l_wvalid, 
      ARADDR     => axi4l_araddr, 
      AWADDR     => axi4l_awaddr, 
      WDATA      => axi4l_wdata, 
      WSTRB      => axi4l_wstrb, 
      ARREADY    => axi4l_arready, 
      AWREADY    => axi4l_awready, 
      BVALID     => axi4l_bvalid, 
      RLAST      => axi4l_rlast,  
      RVALID     => axi4l_rvalid, 
      WREADY     => axi4l_wready, 
      BRESP      => axi4l_bresp,
      RRESP      => axi4l_rresp,
      RDATA      => axi4l_rdata,
      wb_adr     => host_master_out.adr,
      wb_dat_m2s => host_master_out.dat,
      wb_sel     => host_master_out.sel,
      wb_cyc     => host_master_out.cyc,
      wb_stb     => host_master_out.stb,
      wb_we      => host_master_out.we,
      wb_dat_s2m => host_master_in.dat,
      wb_err     => host_master_in.err,
      wb_rty     => host_master_in.rty,
      wb_ack     => host_master_in.ack,
      wb_stall   => host_master_in.stall
      );


-------------------------------------------------------------------------------
-- GTH IBERT
-------------------------------------------------------------------------------

  gen_ibert: if g_with_ibert = true generate
  U_gth_ibert : ibert_ultrascale_gth_0
    port map(
      txn_o => gth_txn,
      txp_o => gth_txp,
      rxoutclk_o => open,
      rxn_i => gth_rxn,
      rxp_i => gth_rxp,
      gtrefclk0_i(0)        => '0',
      gtrefclk1_i(0)        => mgtclk1_225,
      gtnorthrefclk0_i(0)   => '0',
      gtnorthrefclk1_i(0)   => '0',
      gtsouthrefclk0_i(0)   => '0',
      gtsouthrefclk1_i(0)   => '0',
      gtrefclk00_i(0)       => '0',  
      gtrefclk01_i(0)       => '0',   
      gtrefclk10_i(0)       => mgtclk1_225,
      gtrefclk11_i(0)       => mgtclk1_225,
      gtnorthrefclk00_i(0)  => '0',
      gtnorthrefclk10_i(0)  => '0',
      gtnorthrefclk01_i(0)  => '0',
      gtnorthrefclk11_i(0)  => '0',
      gtsouthrefclk00_i(0)  => '0',
      gtsouthrefclk10_i(0)  => '0',
      gtsouthrefclk01_i(0)  => '0',
      gtsouthrefclk11_i(0)  => '0',
      clk                   => mgtclk1_225_gclk
    );
  end generate;

-------------------------------------------------------------------------------
-- GTX PHYs
-------------------------------------------------------------------------------  

  debug_gpi(0) <= to_phys(0).rst;

  gen_phys_no_ibert: if g_with_ibert = false generate
    gen_phys : for i in 0 to c_NUM_PHYS-1 generate

      -- Instantiate GTX with low phase drift calibration
      U_PHY : entity work.wr_gthe4_phy_family7_xilinx_ip
        generic map (
          g_simulation         => f_bool2int(g_simulation),
          g_use_gclk_as_refclk => false )
        port map (
          clk_gth_i => clk_gth(i),
          clk_freerun_i => clk_sys,

          tx_data_i      => to_phys(i).tx_data,
          tx_k_i         => to_phys(i).tx_k,
          tx_disparity_o => from_phys(i).tx_disparity,
          tx_enc_err_o   => from_phys(i).tx_enc_err,
          tx_out_clk_o    => from_phys(i).tx_out_clk,
          rx_rbclk_o     => from_phys(i).rx_clk,
          rx_data_o      => from_phys(i).rx_data,
          rx_k_o         => from_phys(i).rx_k,
          rx_enc_err_o   => from_phys(i).rx_enc_err,
          rx_bitslide_o  => from_phys(i).rx_bitslide,
          rst_i          => to_phys(i).rst,
          loopen_i       => "000",--to_phys(i).loopen,
          pad_txn_o      => gth_txn(i),
          pad_txp_o      => gth_txp(i),
          pad_rxn_i      => gth_rxn(i),
          pad_rxp_i      => gth_rxp(i),
          rdy_o          => from_phys(i).rdy,
          debug_i => x"0000");

      from_phys(i).ref_clk <= clk_ref;

    end generate gen_phys;
  end generate gen_phys_no_ibert;

  gen_terminate_unused_phys : for i in c_NUM_PORTS to c_NUM_PHYS-1 generate
    to_phys(i).tx_data <= (others => '0');
    to_phys(i).tx_k    <= (others => '0');
    to_phys(i).rst     <= '1';
    to_phys(i).loopen  <= '0';
  end generate gen_terminate_unused_phys;


   BUFGMUX_inst : BUFGCTRL
     port map (
       IGNORE0 => '0',
       IGNORE1 => '0',
       CE0 => '1',
       CE1 => '1',
       O   => clk_sys,
       I0  => clk_startup_i,
       I1  => clk_ref,
       S1  => sel_clk_sys,
       S0  => NOT sel_clk_sys);     


  fmc_3v8_enable_o          <= '1';

  U_Ref_Clock_Buffer : IBUFDS_GTE4
      generic map (
        REFCLK_EN_TX_PATH  => '0',
        REFCLK_HROW_CK_SEL => "00",
        REFCLK_ICNTL_RX    => "00")
      port map (
        O     => mgtclk1_225,
        ODIV2 => mgtclk1_225_prebuf,
        CEB   => '0',
        I     => mgtclk1_225_p_i,
        IB    => mgtclk1_225_n_i);

  U_Ref_Clock_Buffer2 : IBUFDS_GTE4
        generic map (
          REFCLK_EN_TX_PATH  => '0',
          REFCLK_HROW_CK_SEL => "00",
          REFCLK_ICNTL_RX    => "00")
        port map (
          O     => mgtclk0_227,
          ODIV2 => mgtclk0_227_prebuf,
          CEB   => '0',
          I     => mgtclk0_227_p_i,
          IB    => mgtclk0_227_n_i);

  --BUFG_GT_3: BUFG_GT
  --  port map (
  --    O       => mgtclk0_227_gclk,
  --    CE      => '1',
  --    CEMASK  => '1', -- '0' -- '1' mask the CE input, ignoring it 
  --    CLR     => '0',
  --    CLRMASK => '1', -- '0' -- '1' mask the CLR input, ignoring it
  --    DIV     => "000",
  --    I       => mgtclk0_227_prebuf);            
  
  --U_Ref_Clock_Buffer3 : IBUFDS_GTE4
  --      generic map (
  --        REFCLK_EN_TX_PATH  => '0',
  --        REFCLK_HROW_CK_SEL => "00",
  --        REFCLK_ICNTL_RX    => "00")
  --      port map (
  --        O     => mgtclk1_227,
  --        CEB   => '0',
  --        I     => mgtclk1_227_p_i,
  --        IB    => mgtclk1_227_n_i);  

  BUFG_GT_2: BUFG_GT
    port map (
      O       => mgtclk1_225_gclk,
      CE      => '1',
      CEMASK  => '1', -- '0' -- '1' mask the CE input, ignoring it 
      CLR     => '0',
      CLRMASK => '1', -- '0' -- '1' mask the CLR input, ignoring it
      DIV     => "000",
      I       => mgtclk1_225_prebuf);     

  --BUFG_GT_3: BUFG_GT
  --  port map (
  --    O       => clk_ref,
  --    CE      => '1',
  --    CEMASK  => '1', -- '0' -- '1' mask the CE input, ignoring it 
  --    CLR     => '0',
  --    CLRMASK => '1', -- '0' -- '1' mask the CLR input, ignoring it
  --    DIV     => "000",
  --    I       => mgtclk1_225_prebuf);       
  
  clk_gth <= (others => mgtclk1_225);
  
  -----------------------------------------------------------------------------
  -- "Bare" top module instantiation
  -----------------------------------------------------------------------------

  U_Real_Top : entity work.scb_top_bare
    generic map (
      g_num_ports       => c_NUM_PORTS,
      g_simulation      => g_simulation,
      g_without_network => false,
      g_with_TRU        => true,
      g_with_TATSU      => true,
      g_with_HWIU       => true,
      g_with_PSTATS     => true,
      g_with_muxed_CS   => false,
      g_with_watchdog   => false,
      g_with_rtu        => true,
      g_with_swc        => true,
      g_with_si57x      => true,
      g_si57x_is_helper => true,
      g_inj_per_EP      => "00" & x"0000",
      g_rtcpu_swap_host_endian => false
      )
    port map (
      sys_rst_n_i         => sys_rst_n_i,
      clk_sys_i           => clk_sys,
      clk_ref_i           => clk_ref,
      clk_aux_i           => clk_aux,
      clk_dmtd_i          => clk_helper_buf,
      --clk_aux_i           => clk_aux_buf,      
      clk_ext_mul_i       => "00",
      clk_ext_mul_locked_i=> '0',
      clk_aux_p_o         => open,
      clk_aux_n_o         => open,
     -- clk_500_o           => clk_500_o,
      cpu_wb_i            => host_master_out,
      cpu_wb_o            => host_master_in,
      cpu_irq_n_o         => cpu_irq_n,
      pps_i               => '0',
      ppsin_term_o        => open,
      pps_o               => open,
      -- Low-jitter daughterboard support
      ljd_dac_main_sync_n_o => open,
      ljd_dac_main_sclk_o   => open,
      ljd_dac_main_data_o   => open,
      ljd_loopback_i        => '0',
      ljd_loopback_o        => open,
      ljd_clk1_en           => open,
      ljd_clk2_en           => open,
      ljd_detected_o        => open,
      ljd_osc_freq_i        => "000",
      ljd_pll_mosi_o        => open,
      ljd_pll_miso_i        => '0',
      ljd_pll_sck_o         => open,
      ljd_pll_cs_n_o        => open,
      ljd_pll_sync_n_o      => open,
      ljd_pll_reset_n_o     => open,
      ljd_pll_locked_i      => '0',

      dac_helper_load_o  => dac_helper_load,
      dac_helper_value_o => dac_helper_value,
      dac_main_load_o  => dac_main_load,
      dac_main_value_o  => dac_main_value,

      
      pll_status_i        => '0',   --unused for now, use one of hmc gpios?
      pll_mosi_o          => hmc_mosi_o,
      pll_miso_i          => hmc_miso_i,
      pll_sck_o           => hmc_sclk_o,
      pll_cs_n_o          => hmc_slen_n_o,
      pll_sync_n_o        => open,  --unused for now
      pll_reset_n_o       => hmc_reset_n_o,
      uart_txd_o          => uart_txd_o,
      uart_rxd_i          => uart_rxd_i,
      clk_en_o            => open,
      clk_sel_o           => open,
--    uart_sel_o          => uart_sel_o,
      clk_dmtd_divsel_o   => open,
      gpio_i              => x"00000000",
      phys_o              => to_phys(c_NUM_PORTS-1 downto 0),
      phys_i              => from_phys(c_NUM_PORTS-1 downto 0),
--      led_link_o          => led_link_o,
--      led_act_o           => led_act_o(c_NUM_PORTS-1 downto 0),
      i2c_scl_oen_o   => i2c_scl_oen,
      i2c_scl_o       => i2c_scl_out,
      i2c_scl_i       => i2c_scl_in,
      i2c_sda_oen_o   => i2c_sda_oen,
      i2c_sda_o       => i2c_sda_out,
      i2c_sda_i       => i2c_sda_in,

      --sfp i2c ports
      sfp_i2c_scl_oen_o   => sfp_scl_oen,
      sfp_i2c_scl_o       => sfp_scl_out,
      sfp_i2c_scl_i       => sfp_scl_in,
      sfp_i2c_sda_oen_o   => sfp_sda_oen,
      sfp_i2c_sda_o       => sfp_sda_out,
      sfp_i2c_sda_i       => sfp_sda_in,

      mb_fan1_pwm_o   => open,
      mb_fan2_pwm_o   => open,

      sel_clk_sys_o => sel_clk_sys,

      rts_si57x_scl_oen_o => rts_si57x_scl_oen,
      rts_si57x_sda_oen_o => rts_si57x_sda_oen,
      rts_si57x_scl_i => rts_si57x_scl_in,
      rts_si57x_sda_i => rts_si57x_sda_in,

      aux_scl_i => aux_scl_in,
      aux_sda_i => aux_sda_in,
      aux_scl_oen_o => aux_scl_oen,
      aux_sda_oen_o => aux_sda_oen,
      
      rts_gpio_o => rts_gpio_out,
      rts_gpio_i => rts_gpio_in,

      dbg_gpio_i => debug_gpi,
      dbg_gpio_o => debug_gpo
      );


  --mgt225_tx_n_o <= gth_txn(3 downto 0); --FMC1
  --mgt225_tx_p_o <= gth_txp(3 downto 0);
  --mgt227_tx_n_o <= gth_txn(7 downto 4); --FMC2
  --mgt227_tx_p_o <= gth_txp(7 downto 4);

  mgt227_tx_n_o <= gth_txn; --FMC2
  mgt227_tx_p_o <= gth_txp;
  gth_rxn       <= mgt227_rx_n_i; --FMC2
  gth_rxp       <= mgt227_rx_p_i;


  --gth_rxn(3 downto 0) <= mgt225_rx_n_i; --FMC1
  --gth_rxp(3 downto 0) <= mgt225_rx_p_i; 
  --gth_rxn(7 downto 4) <= mgt227_rx_n_i; --FMC2
  --gth_rxp(7 downto 4) <= mgt227_rx_p_i;
  
  IOBUF_1 : IOBUF
   port map (
     O  => rts_si57x_sda_in,
     IO => si57x_sda_b,
     I  => '0',
     T  => rts_si57x_sda_oen); 

  IOBUF_2 : IOBUF
   port map (
     O  => rts_si57x_scl_in,
     IO => si57x_scl_b,
     I  => '0',
     T  => rts_si57x_scl_oen);

  IOBUF_3 : IOBUF
   port map (
     O  => aux_scl_in,
     IO => i2c_scl_b,
     I  => '0',
     T =>  aux_scl_oen); 

  IOBUF_4 : IOBUF
   port map (
     O  => aux_sda_in,
     IO => i2c_sda_b,
     I  => '0',
     T => aux_sda_oen); 

  gen_sfp_i2c_iobufs: for i in 0 to c_num_sfp_ports-1 generate 

    SFP_SCL_IOBUF: IOBUF
    port map 
    (
      O  => sfp_scl_in(i),
      IO => sfp_scl_b(i),
      I  => sfp_scl_out(i),
      T => sfp_scl_oen(i)
    );    

    SFP_SDA_IOBUF: IOBUF 
    port map 
    (
      O  => sfp_sda_in(i),
      IO => sfp_sda_b(i),
      I  => sfp_sda_out(i),
      T => sfp_sda_oen(i)
    );   

  end generate;

  IOBUF_HMC_SDATA : IOBUF
  port map (
    O  => hmc_miso_i,
    IO => hmc_sdata_b,
    I  => '0',
    T => hmc_mosi_o
  );

  hmc_sync_o <= '0';

  OBUF_HMC_SYNC: OBUFDS 
  port map 
  (
    I   => hmc_sync_o,
    O   => hmc_sync_p_o,
    OB  => hmc_sync_n_o
  );

  hmc_reset_o <= not hmc_reset_n_o;
  
  U_WR_DACs : entity work.spec_serial_dac_arb
    generic map(
      g_invert_sclk    => false,
      g_num_extra_bits => 8)
    port map(
      clk_i       => clk_sys,
      rst_n_i     => sys_rst_n_i,
      val1_i      => dac_main_value,
      load1_i     => dac_main_load,
      val2_i      => dac_helper_value,
      load2_i     => dac_helper_load,
      dac_cs_n_o  => wr_dac1_sync_n_o,
      dac_clr_n_o => open,
      dac_sclk_o  => wr_dac1_sclk_o,
      dac_din_o   => wr_dac1_din_o);

end Behavioral;


