-------------------------------------------------------------------------------
-- Title      : WR Switch v4 bringup top level
-- Project    : White Rabbit Switch
-------------------------------------------------------------------------------
-- File       : scb_top_synthesis.vhd
-- Author     : Tomasz Wlostowski, Maciej Lipinski, Grzegorz Daniluk, Harvey Leicester
-- Company    : CERN BE-CO-HT
-- Created    : 2012-03-07
-- Last update: 2020-08-18
-- Platform   : FPGA-generic
-- Standard   : VHDL
-------------------------------------------------------------------------------
-- Description:
-- WR Switch v4 hardware bringup version top synthesis level.
-------------------------------------------------------------------------------
--
-- Copyright (c) 2012 - 2014 CERN / BE-CO-HT
--
-- This source file is free software; you can redistribute it
-- and/or modify it under the terms of the GNU Lesser General
-- Public License as published by the Free Software Foundation;
-- either version 2.1 of the License, or (at your option) any
-- later version.
--
-- This source is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied
-- warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-- PURPOSE.  See the GNU Lesser General Public License for more
-- details.
--
-- You should have received a copy of the GNU Lesser General
-- Public License along with this source; if not, download it
-- from http://www.gnu.org/licenses/lgpl-2.1.html
--
-------------------------------------------------------------------------------

library ieee;
use ieee.STD_LOGIC_1164.all;
use ieee.numeric_std.all;

library work;
use work.wishbone_pkg.all;
use work.gencores_pkg.all;
use work.wr_fabric_pkg.all;
use work.endpoint_pkg.all;
use work.wr_txtsu_pkg.all;
use work.wrsw_top_pkg.all;
use work.wrsw_shared_types_pkg.all;
use work.ps_wrapper;

library UNISIM;
use UNISIM.vcomponents.all;


entity wrsv4_bringup is
  generic(
    g_simulation          : boolean := false;
    g_with_ibert          : boolean := true
    );
  port (
    
    fan1_present_i              : in std_logic;       
    fan2_present_i              : in std_logic;
    wr_clk_p_i                  : in std_logic;
    wr_clk_n_i                  : in std_logic;
    ref_clk_p_i                 : in std_logic;
    ref_clk_n_i                 : in std_logic;
    helper_clk_p_i              : in std_logic;
    helper_clk_n_i              : in std_logic;
    aux_clk_p_i                 : in std_logic;
    aux_clk_n_i                 : in std_logic;
    lcd_bbutton_1_i             : in std_logic;
    lcd_bbutton_2_i             : in std_logic;
    
    psu_alert_1v8_i             : in std_logic;
    term_en_o                   : out std_logic;
    pps_out_o                   : out std_logic;
    aux_out_o                   : out std_logic;
    pll_gm_csb_o                : out std_logic;
    pll_gm_sdi_o                : out std_logic;
    pll_gm_sck_o                : out std_logic;
    pll_gm_sync_o               : out std_logic;
    hmc_sync_o                  : out std_logic;
    hmc_sdata_b                 : inout std_logic;
    hmc_slen_o                  : out std_logic;
    hmc_sclk_o                  : out std_logic;
    hmc_reset_o                 : out std_logic;
    fmc_pwr_en_o                : out std_logic;
    fmc_pg_i                    : in std_logic;
    aux_clk_fpga_p_i            : in std_logic;
    aux_clk_fpga_n_i            : in std_logic;
    lmx_gm_clk_p_i              : in std_logic;
    lmx_gm_clk_n_i              : in std_logic;
    ext_gm_fpga_p_i             : in std_logic;
    ext_gm_fpga_n_i             : in std_logic;
    pll_oscout1_clk_p_i         : in std_logic;
    pll_oscout1_clk_n_i         : in std_logic;
    gp_button_0_i               : in std_logic;
    gp_button_1_i               : in std_logic;
    
    pps_in_i                    : in std_logic;
    gm_pwr_en_o                 : out std_logic;
    clk_100mhz_out_clk_sel_o    : out std_logic;
    gth_clk1_sel_o              : out std_logic;
    pll_gm_rampdir_o            : out std_logic;
    pll_gm_rampclk_o            : out std_logic;
    pll_gm_muxout_i             : in std_logic;
    pll_gm_sysrefreq_o          : out std_logic;
    hmc_gpio1_b                 : inout std_logic;
    hmc_gpio2_b                 : inout std_logic;
    oled_ctrl_i2c_scl_b         : inout std_logic;
    oled_ctrl_i2c_sda_b         : inout std_logic;
    pll_clk_sel_o               : out std_logic;
    dac_sync_n_o                : out std_logic;
    dac_din_o                   : out std_logic;
    dac_sclk_o                  : out std_logic;
    uart_rxd_i                  : in std_logic;
    uart_txd_o                  : out std_logic;
    scl_aux_b                   : inout std_logic;
    sda_aux_b                   : inout std_logic;
    scl_helper_b                : inout std_logic;
    sda_helper_b                : inout std_logic;
    prsnt_m2c_l_i               : in std_logic;
    pg_m2c_i                    : in std_logic;
    
    --sfps
    lt_sfp_scl_b                : inout std_logic_vector(2 downto 0);
    lt_sfp_sda_b                : inout std_logic_vector(2 downto 0);  
    lt_sfp_los_i                : in std_logic_vector(2 downto 0);    
    lt_sfp_led_mode_o           : out std_logic_vector(2 downto 0);
    lt_sfp_led_link_o           : out std_logic_vector(2 downto 0);
    lt_sfp_led_synced_o         : out std_logic_vector(2 downto 0);
    lt_sfp_led_activity_o       : out std_logic_vector(2 downto 0);
    lt_sfp_tx_disable_o         : out std_logic_vector(2 downto 0);
    lt_sfp_tx_fault_i           : in std_logic_vector(2 downto 0);
    lt_sfp_detect_i             : in std_logic_vector(2 downto 0);
  
    lct_sfp_scl_b               : inout std_logic_vector(2 downto 0);    
    lct_sfp_sda_b               : inout std_logic_vector(2 downto 0);
    lct_sfp_los_i               : in std_logic_vector(2 downto 0);
    lct_sfp_led_mode_o          : out std_logic_vector(2 downto 0);
    lct_sfp_led_link_o          : out std_logic_vector(2 downto 0);
    lct_sfp_led_synced_o        : out std_logic_vector(2 downto 0);
    lct_sfp_led_activity_o      : out std_logic_vector(2 downto 0);
    lct_sfp_tx_disable_o        : out std_logic_vector(2 downto 0);
    lct_sfp_tx_fault_i          : in std_logic_vector(2 downto 0);
    lct_sfp_detect_i            : in std_logic_vector(2 downto 0);    

    rct_sfp_scl_b               : inout std_logic_vector(2 downto 0);
    rct_sfp_sda_b               : inout std_logic_vector(2 downto 0);
    rct_sfp_los_i               : in std_logic_vector(2 downto 0);
    rct_sfp_led_mode_o          : out std_logic_vector(2 downto 0);
    rct_sfp_led_link_o          : out std_logic_vector(2 downto 0);
    rct_sfp_led_synced_o        : out std_logic_vector(2 downto 0);
    rct_sfp_led_activity_o      : out std_logic_vector(2 downto 0);
    rct_sfp_tx_disable_o        : out std_logic_vector(2 downto 0);
    rct_sfp_tx_fault_i          : in std_logic_vector(2 downto 0);
    rct_sfp_detect_i            : in std_logic_vector(2 downto 0);    

    rt_sfp_scl_b                : inout std_logic_vector(2 downto 0);
    rt_sfp_sda_b                : inout std_logic_vector(2 downto 0);
    rt_sfp_los_i                : in std_logic_vector(2 downto 0);
    rt_sfp_led_mode_o           : out std_logic_vector(2 downto 0);
    rt_sfp_led_link_o           : out std_logic_vector(2 downto 0);
    rt_sfp_led_synced_o         : out std_logic_vector(2 downto 0);
    rt_sfp_led_activity_o       : out std_logic_vector(2 downto 0);
    rt_sfp_tx_disable_o         : out std_logic_vector(2 downto 0);
    rt_sfp_tx_fault_i           : in std_logic_vector(2 downto 0);
    rt_sfp_detect_i             : in std_logic_vector(2 downto 0);          

    lb_sfp_scl_b                : inout std_logic_vector(2 downto 0);
    lb_sfp_sda_b                : inout std_logic_vector(2 downto 0);  
    lb_sfp_los_i                : in std_logic_vector(2 downto 0);
    lb_sfp_led_mode_o           : out std_logic_vector(2 downto 0);
    lb_sfp_led_link_o           : out std_logic_vector(2 downto 0);
    lb_sfp_led_synced_o         : out std_logic_vector(2 downto 0);
    lb_sfp_led_activity_o       : out std_logic_vector(2 downto 0);
    lb_sfp_tx_disable_o         : out std_logic_vector(2 downto 0);
    lb_sfp_tx_fault_i           : in std_logic_vector(2 downto 0);
    lb_sfp_detect_i             : in std_logic_vector(2 downto 0);      

    lcb_sfp_scl_b               : inout std_logic_vector(2 downto 0);
    lcb_sfp_sda_b               : inout std_logic_vector(2 downto 0);  
    lcb_sfp_los_i               : in std_logic_vector(2 downto 0);
    lcb_sfp_led_mode_o          : out std_logic_vector(2 downto 0);
    lcb_sfp_led_link_o          : out std_logic_vector(2 downto 0);
    lcb_sfp_led_synced_o        : out std_logic_vector(2 downto 0);
    lcb_sfp_led_activity_o      : out std_logic_vector(2 downto 0);
    lcb_sfp_tx_disable_o        : out std_logic_vector(2 downto 0);
    lcb_sfp_tx_fault_i          : in std_logic_vector(2 downto 0);
    lcb_sfp_detect_i            : in std_logic_vector(2 downto 0);      

    rcb_sfp_scl_b               : inout std_logic_vector(2 downto 0);
    rcb_sfp_sda_b               : inout std_logic_vector(2 downto 0);  
    rcb_sfp_los_i               : in std_logic_vector(2 downto 0); 
    rcb_sfp_led_mode_o          : out std_logic_vector(2 downto 0);
    rcb_sfp_led_link_o          : out std_logic_vector(2 downto 0);
    rcb_sfp_led_synced_o        : out std_logic_vector(2 downto 0);
    rcb_sfp_led_activity_o      : out std_logic_vector(2 downto 0);
    rcb_sfp_tx_disable_o        : out std_logic_vector(2 downto 0);
    rcb_sfp_tx_fault_i          : in std_logic_vector(2 downto 0);
    rcb_sfp_detect_i            : in std_logic_vector(2 downto 0);        

    rb_sfp_scl_b                : inout std_logic_vector(2 downto 0);
    rb_sfp_sda_b                : inout std_logic_vector(2 downto 0);  
    rb_sfp_los_i                : in std_logic_vector(2 downto 0);
    rb_sfp_led_mode_o           : out std_logic_vector(2 downto 0);
    rb_sfp_led_link_o           : out std_logic_vector(2 downto 0);
    rb_sfp_led_synced_o         : out std_logic_vector(2 downto 0);
    rb_sfp_led_activity_o       : out std_logic_vector(2 downto 0);
    rb_sfp_tx_disable_o         : out std_logic_vector(2 downto 0);
    rb_sfp_tx_fault_i           : in std_logic_vector(2 downto 0);
    rb_sfp_detect_i             : in std_logic_vector(2 downto 0);        
  
    --gth
    gth224_tx_p_o               : out std_logic_vector(3 downto 0);
    gth224_tx_n_o               : out std_logic_vector(3 downto 0);
    gth224_rx_p_i               : in std_logic_vector(3 downto 0);
    gth224_rx_n_i               : in std_logic_vector(3 downto 0);
    gth224_clk0_p_i             : in std_logic;
    gth224_clk0_n_i             : in std_logic;    
    gth224_clk1_p_i             : in std_logic;
    gth224_clk1_n_i             : in std_logic;
    
    gth225_tx_p_o               : out std_logic_vector(3 downto 0);
    gth225_tx_n_o               : out std_logic_vector(3 downto 0);
    gth225_rx_p_i               : in std_logic_vector(3 downto 0);
    gth225_rx_n_i               : in std_logic_vector(3 downto 0);
    gth225_clk0_p_i             : in std_logic;  
    gth225_clk0_n_i             : in std_logic;      
    gth225_clk1_p_i             : in std_logic;  
    gth225_clk1_n_i             : in std_logic;  
    
    gth226_tx_p_o               : out std_logic_vector(3 downto 0);
    gth226_tx_n_o               : out std_logic_vector(3 downto 0);
    gth226_rx_p_i               : in std_logic_vector(3 downto 0);
    gth226_rx_n_i               : in std_logic_vector(3 downto 0);
    gth226_clk0_p_i             : in std_logic;  
    gth226_clk0_n_i             : in std_logic;      
    gth226_clk1_p_i             : in std_logic;  
    gth226_clk1_n_i             : in std_logic;  
    
    gth227_tx_p_o               : out std_logic_vector(3 downto 0);                 
    gth227_tx_n_o               : out std_logic_vector(3 downto 0);
    gth227_rx_p_i               : in std_logic_vector(3 downto 0);
    gth227_rx_n_i               : in std_logic_vector(3 downto 0);
    gth227_clk0_p_i             : in std_logic;
    gth227_clk0_n_i             : in std_logic;
    gth227_clk1_p_i             : in std_logic;
    gth227_clk1_n_i             : in std_logic;
    
    gth228_tx_p_o               : out std_logic_vector(3 downto 0);
    gth228_tx_n_o               : out std_logic_vector(3 downto 0);
    gth228_rx_p_i               : in std_logic_vector(3 downto 0);
    gth228_rx_n_i               : in std_logic_vector(3 downto 0);
    gth228_clk0_p_i             : in std_logic; 
    gth228_clk0_n_i             : in std_logic;     
    gth228_clk1_p_i             : in std_logic; 
    gth228_clk1_n_i             : in std_logic; 
    
    gth229_tx_p_o               : out std_logic_vector(3 downto 0);
    gth229_tx_n_o               : out std_logic_vector(3 downto 0);
    gth229_rx_p_i               : in std_logic_vector(3 downto 0);
    gth229_rx_n_i               : in std_logic_vector(3 downto 0);
    gth229_clk0_p_i             : in std_logic;  
    gth229_clk0_n_i             : in std_logic;      
    gth229_clk1_p_i             : in std_logic;  
    gth229_clk1_n_i             : in std_logic  
    
    --gth230_tx_p_o               : out std_logic_vector(3 downto 0);
    --gth230_tx_n_o               : out std_logic_vector(3 downto 0);
    --gth230_rx_p_i               : in std_logic_vector(3 downto 0);
    --gth230_rx_n_i               : in std_logic_vector(3 downto 0);
    --gth230_clk0_p_i             : in std_logic;
    --gth230_clk0_n_i             : in std_logic;    
    --gth230_clk1_p_i             : in std_logic;
    --gth230_clk1_n_i             : in std_logic;
    
    --gth231_tx_p_o               : out std_logic_vector(3 downto 0);
    --gth231_tx_n_o               : out std_logic_vector(3 downto 0);
    --gth231_rx_p_i               : in std_logic_vector(3 downto 0);
    --gth231_rx_n_i               : in std_logic_vector(3 downto 0);
    --gth231_clk0_p_i             : in std_logic;
    --gth231_clk0_n_i             : in std_logic;
    --gth231_clk1_p_i             : in std_logic;
    --gth231_clk1_n_i             : in std_logic;
    
    --gty
    --gty128_tx_p_o               : out std_logic_vector(3 downto 0);   
    --gty128_tx_n_o               : out std_logic_vector(3 downto 0); 
    --gty128_rx_p_i               : in std_logic_vector(3 downto 0); 
    --gty128_rx_n_i               : in std_logic_vector(3 downto 0); 
    --gty128_clk0_p_i             : in std_logic; 
    --gty128_clk0_n_i             : in std_logic; 
    --gty128_clk1_p_i             : in std_logic;
    --gty128_clk1_n_i             : in std_logic;
    
    --gty129_tx_p_o               : out std_logic_vector(3 downto 0);            
    --gty129_tx_n_o               : out std_logic_vector(3 downto 0);     
    --gty129_rx_p_i               : in std_logic_vector(3 downto 0);   
    --gty129_rx_n_i               : in std_logic_vector(3 downto 0);
    --gty129_clk0_p_i             : in std_logic;   
    --gty129_clk0_n_i             : in std_logic;   
    --gty129_clk1_p_i             : in std_logic;    
    --gty129_clk1_n_i             : in std_logic;  
    
    --gty130_tx_p_o               : out std_logic_vector(3 downto 0);  
    --gty130_tx_n_o               : out std_logic_vector(3 downto 0);
    --gty130_rx_p_i               : in std_logic_vector(3 downto 0);
    --gty130_rx_n_i               : in std_logic_vector(3 downto 0);
    --gty130_clk0_p_i             : in std_logic;   
    --gty130_clk0_n_i             : in std_logic;   
    --gty130_clk1_p_i             : in std_logic;   
    --gty130_clk1_n_i             : in std_logic;   
    
    --gty131_tx_p_o               : out std_logic_vector(3 downto 0);       
    --gty131_tx_n_o               : out std_logic_vector(3 downto 0);
    --gty131_rx_p_i               : in std_logic_vector(3 downto 0);
    --gty131_rx_n_i               : in std_logic_vector(3 downto 0);
    --gty131_clk0_p_i             : in std_logic;    
    --gty131_clk0_n_i             : in std_logic;    
    --gty131_clk1_p_i             : in std_logic;    
    --gty131_clk1_n_i             : in std_logic    
  );
end wrsv4_bringup;

architecture Behavioral of wrsv4_bringup is
  
  constant c_NUM_PHYS         : integer := 18;
  constant c_NUM_PORTS        : integer := 18;
  constant c_NUM_GTH_QUADS    : integer := 6;
  constant c_NUM_GTH_REFCLKS  : integer := 6;

  function f_bool2int(x : boolean) return integer is
  begin
    if(x) then
      return 1;
    else
      return 0;
    end if;
  end f_bool2int;


  signal to_phys   : t_phyif_output_array(c_NUM_PHYS-1 downto 0);
  signal from_phys : t_phyif_input_array(c_NUM_PHYS-1 downto 0);

  signal clk_aux      : std_logic;
  signal clk_gtx      : std_logic_vector(c_NUM_PHYS-1 downto 0);

  signal axi4l_aclk     : std_logic;
  signal axi4l_aresetn  : std_logic;
  signal axi4l_arvalid :  std_logic;
  signal axi4l_awvalid :  std_logic;
  signal axi4l_bready  :  std_logic;
  signal axi4l_rready  :  std_logic;
  signal axi4l_wlast   :  std_logic := '1';
  signal axi4l_wvalid  :  std_logic;
  signal axi4l_araddr  :  std_logic_vector (31 downto 0);
  signal axi4l_awaddr  :  std_logic_vector (31 downto 0);
  signal axi4l_wdata   :  std_logic_vector (31 downto 0);
  signal axi4l_wstrb   :  std_logic_vector (3 downto 0);
  signal axi4l_arready :  std_logic;
  signal axi4l_awready :  std_logic;
  signal axi4l_bvalid  :  std_logic;
  signal axi4l_rlast   :  std_logic;
  signal axi4l_rvalid  :  std_logic;
  signal axi4l_wready  :  std_logic;
  signal axi4l_bresp   :  std_logic_vector (1 downto 0);
  signal axi4l_rresp   :  std_logic_vector (1 downto 0);
  signal axi4l_rdata   :  std_logic_vector (31 downto 0);  

  signal sys_rst_n_i    : std_logic;         -- global reset
  signal clk_startup_i  : std_logic; -- startup clock, 100 MHz (PCIe core)  
  
  component scb_top_bare
    generic (
      g_num_ports       : integer;
      g_simulation      : boolean;
      g_without_network : boolean;
      g_with_TRU        : boolean;
      g_with_TATSU      : boolean;
      g_with_HWIU       : boolean;
      g_with_PSTATS     : boolean;
      g_with_muxed_CS   : boolean;
      g_with_watchdog   : boolean;
      g_with_rtu        : boolean;       
      g_with_swc        : boolean;
      g_with_si57x      : boolean;
      g_si57x_is_helper : boolean;
      g_inj_per_EP      : std_logic_vector(17 downto 0);
      g_phy_lpcalib     : t_bool_array(0 to 17) := c_BOOL_FALSE_ARRAY;
      g_rtcpu_swap_host_endian : boolean
    );
    port (
      sys_rst_n_i           : in  std_logic;
      clk_startup_i         : in  std_logic;
      clk_ref_i             : in  std_logic;
      clk_dmtd_i            : in  std_logic;
      clk_aux_i             : in  std_logic;
      clk_ext_mul_i         : in  std_logic_vector(1 downto 0);
      clk_ext_mul_locked_i  : in  std_logic;
      clk_aux_p_o           : out std_logic;
      clk_aux_n_o           : out std_logic;
      clk_500_o             : out std_logic;
      clk_sys_o             : out std_logic;
      cpu_wb_i              : in  t_wishbone_slave_in;
      cpu_wb_o              : out t_wishbone_slave_out;
      cpu_irq_n_o           : out std_logic;
      pps_i                 : in  std_logic;
      ppsin_term_o          : out std_logic;
      pps_o                 : out std_logic;
      ljd_dac_main_sync_n_o : out std_logic;
      ljd_dac_main_sclk_o   : out std_logic;
      ljd_dac_main_data_o   : out std_logic;
      ljd_loopback_i        : in  std_logic;
      ljd_loopback_o        : out std_logic;
      ljd_clk1_en           : out std_logic;
      ljd_clk2_en           : out std_logic;
      ljd_detected_o        : out std_logic;
      ljd_osc_freq_i        : in  std_logic_vector (2 downto 0);
      ljd_pll_mosi_o        : out std_logic;
      ljd_pll_miso_i        : in  std_logic;
      ljd_pll_sck_o         : out std_logic;
      ljd_pll_cs_n_o        : out std_logic;
      ljd_pll_sync_n_o      : out std_logic;
      ljd_pll_reset_n_o     : out std_logic;
      ljd_pll_locked_i      : in  std_logic;
      pll_status_i          : in  std_logic;
      pll_mosi_o            : out std_logic;
      pll_miso_i            : in  std_logic;
      pll_sck_o             : out std_logic;
      pll_cs_n_o            : out std_logic;
      pll_sync_n_o          : out std_logic;
      pll_reset_n_o         : out std_logic;
      uart_txd_o            : out std_logic;
      uart_rxd_i            : in  std_logic;
      clk_en_o              : out std_logic;
      clk_sel_o             : out std_logic;
      uart_sel_o            : out std_logic;
      clk_dmtd_divsel_o     : out std_logic;
      phys_o                : out t_phyif_output_array(g_num_ports-1 downto 0);
      phys_i                : in  t_phyif_input_array(g_num_ports-1 downto 0);
      led_link_o            : out std_logic_vector(g_num_ports-1 downto 0);
      led_act_o             : out std_logic_vector(g_num_ports-1 downto 0);
      gpio_o                : out std_logic_vector(31 downto 0);
      gpio_i                : in  std_logic_vector(31 downto 0);
      i2c_scl_oen_o         : out std_logic_vector(2 downto 0);
      i2c_scl_o             : out std_logic_vector(2 downto 0);
      i2c_scl_i             : in  std_logic_vector(2 downto 0) := "111";
      i2c_sda_oen_o         : out std_logic_vector(2 downto 0);
      i2c_sda_o             : out std_logic_vector(2 downto 0);
      i2c_sda_i             : in  std_logic_vector(2 downto 0) := "111";
      mb_fan1_pwm_o         : out std_logic;
      mb_fan2_pwm_o         : out std_logic;
      spll_dbg_o            : out std_logic_vector(5 downto 0);

      rts_si57x_scl_oen_o   : out std_logic;
      rts_si57x_sda_oen_o   : out std_logic;
      rts_si57x_scl_i       : in std_logic;
      rts_si57x_sda_i       : in std_logic;

      --sfp i2c ports
      sfp_i2c_scl_oen_o     : out std_logic_vector(g_num_ports-1 downto 0);
      sfp_i2c_scl_o         : out std_logic_vector(g_num_ports-1 downto 0);
      sfp_i2c_scl_i         : in std_logic_vector(g_num_ports-1 downto 0);
      sfp_i2c_sda_oen_o     : out std_logic_vector(g_num_ports-1 downto 0);
      sfp_sda_o             : out std_logic_vector(g_num_ports-1 downto 0);
      sfp_sda_i             : out std_logic_vector(g_num_ports-1 downto 0);
      sfp_tx_disable_o      : out std_logic_vector(g_num_ports-1 downto 0);

      dbg_gpio_o            : out std_logic_vector(7 downto 0);
      dbg_gpio_i            : in std_logic_vector(7 downto 0)
      );
  end component;

  component ibert_ultrascale_gth_0
    port (
      txn_o : out std_logic_vector((4*c_NUM_GTH_QUADS)-1 DOWNTO 0);
      txp_o : out std_logic_vector((4*c_NUM_GTH_QUADS)-1 DOWNTO 0);
      rxn_i : in std_logic_vector((4*c_NUM_GTH_QUADS)-1 DOWNTO 0);
      rxp_i : in std_logic_vector((4*c_NUM_GTH_QUADS)-1 DOWNTO 0);
      rxoutclk_o  : out std_logic_vector((4*c_NUM_GTH_QUADS)-1 DOWNTO 0);
      gtrefclk0_i : in std_logic_vector(c_NUM_GTH_REFCLKS-1 DOWNTO 0);
      gtrefclk1_i : in std_logic_vector(c_NUM_GTH_REFCLKS-1 DOWNTO 0);
      gtnorthrefclk0_i : in std_logic_vector(c_NUM_GTH_QUADS-1 DOWNTO 0);
      gtnorthrefclk1_i : in std_logic_vector(c_NUM_GTH_QUADS-1 DOWNTO 0);
      gtsouthrefclk0_i : in std_logic_vector(c_NUM_GTH_QUADS-1 DOWNTO 0);
      gtsouthrefclk1_i : in std_logic_vector(c_NUM_GTH_QUADS-1 DOWNTO 0);
      gtrefclk00_i : in std_logic_vector(c_NUM_GTH_QUADS-1 DOWNTO 0);
      gtrefclk10_i : in std_logic_vector(c_NUM_GTH_QUADS-1 DOWNTO 0);
      gtrefclk01_i : in std_logic_vector(c_NUM_GTH_QUADS-1 DOWNTO 0);
      gtrefclk11_i : in std_logic_vector(c_NUM_GTH_QUADS-1 DOWNTO 0);
      gtnorthrefclk00_i : in std_logic_vector(c_NUM_GTH_QUADS-1 DOWNTO 0);
      gtnorthrefclk10_i : in std_logic_vector(c_NUM_GTH_QUADS-1 DOWNTO 0);
      gtnorthrefclk01_i : in std_logic_vector(c_NUM_GTH_QUADS-1 DOWNTO 0);
      gtnorthrefclk11_i : in std_logic_vector(c_NUM_GTH_QUADS-1 DOWNTO 0);
      gtsouthrefclk00_i : in std_logic_vector(c_NUM_GTH_QUADS-1 DOWNTO 0);
      gtsouthrefclk10_i : in std_logic_vector(c_NUM_GTH_QUADS-1 DOWNTO 0);
      gtsouthrefclk01_i : in std_logic_vector(c_NUM_GTH_QUADS-1 DOWNTO 0);
      gtsouthrefclk11_i : in std_logic_vector(c_NUM_GTH_QUADS-1 DOWNTO 0);
      clk : in std_logic 
    );
  end component;

  signal clk_20m_vcxo_buf : std_logic;
  signal clk_dmtd_bufin   : std_logic;
  signal clk_dmtd         : std_logic;
  signal clk_dmtd_fbout   : std_logic;
  signal clk_dmtd_fbin    : std_logic;
  signal dmtd_pll_locked  : std_logic;

  signal rst_sys          : std_logic;
  signal clk_sys          : std_logic;

  signal sel_clk_sys      : std_logic;
  
  signal clk_swcore_fbout, clk_swcore_fbin, clk_swcore_bufin : std_logic;
  signal swcore_pll_locked, swcore_pll_rst : std_logic;

  signal host_master_out  : t_wishbone_master_out;
  signal host_master_in   : t_wishbone_master_in;

  attribute mark_debug : string;
  attribute mark_debug of host_master_in :signal is "true";
  attribute mark_debug of host_master_out :signal is "true";
  
  signal gth_refclk         : std_logic_vector(c_NUM_PHYS-1 downto 0);
  signal gth_txoutclk       : std_logic_vector(c_NUM_PHYS-1 downto 0);
  signal clk_ref            : std_logic;

  signal cpu_irq_n          : std_logic;

  signal i2c_scl_oen        : std_logic_vector(2 downto 0);
  signal i2c_sda_oen        : std_logic_vector(2 downto 0);
  signal i2c_scl_out        : std_logic_vector(2 downto 0);
  signal i2c_sda_out        : std_logic_vector(2 downto 0);
  signal i2c_scl_in         : std_logic_vector(2 downto 0);
  signal i2c_sda_in         : std_logic_vector(2 downto 0);

  signal rts_si549_scl_oen  : std_logic;
  signal rts_si549_sda_oen  : std_logic;
  signal rts_si549_scl_in   : std_logic;
  signal rts_si549_sda_in   : std_logic;
  signal rts_gpio_out       : std_logic_vector(7 downto 0);
  signal rts_gpio_in        : std_logic_vector(7 downto 0);

  signal hmc_miso_i         : std_logic;
  signal hmc_mosi_o         : std_logic;
  signal hmc_reset_n_o      : std_logic;
  
  signal gth_txp, gth_rxp, gth_txn, gth_rxn : std_logic_vector((c_NUM_GTH_QUADS*4)-1 downto 0);

  signal clk_gth : std_logic_vector((c_NUM_GTH_QUADS*4)-1 downto 0);

  signal dac_main_value, dac_helper_value : std_logic_vector(15 downto 0);
  signal dac_main_load, dac_helper_load : std_logic;

  signal mgtclk0_224        : std_logic;
  signal mgtclk0_224_prebuf : std_logic;
  signal mgtclk0_224_gclk   : std_logic;
  signal mgtclk0_225        : std_logic;
  signal mgtclk0_226        : std_logic;
  signal mgtclk0_227        : std_logic;
  signal mgtclk0_228        : std_logic;
  signal mgtclk0_229        : std_logic;

  signal gth_qrefclk0       : std_logic_vector(c_NUM_GTH_QUADS-1 downto 0);
  signal gth_qrefclk00      : std_logic_vector(c_NUM_GTH_QUADS-1 downto 0);
  signal gth_qrefclk01      : std_logic_vector(c_NUM_GTH_QUADS-1 downto 0);
  
  signal gpio_in            : std_logic_vector(31 downto 0) := (others => '0');

  type t_sfp_slv_quad_array is array (0 to c_NUM_GTH_QUADS-1) of std_logic_vector(3 downto 0);

  signal sfp_scl_oen_array  : t_sfp_slv_quad_array;
  signal sfp_sda_oen_array  : t_sfp_slv_quad_array;
  signal sfp_scl_out_array  : t_sfp_slv_quad_array;
  signal sfp_sda_out_array  : t_sfp_slv_quad_array;
  signal sfp_scl_in_array   : t_sfp_slv_quad_array;
  signal sfp_sda_in_array   : t_sfp_slv_quad_array;

  signal sfp_sda_in         : std_logic_vector((c_NUM_GTH_QUADS*4)-1 downto 0);
  signal sfp_scl_in         : std_logic_vector((c_NUM_GTH_QUADS*4)-1 downto 0);
  signal sfp_sda_out        : std_logic_vector((c_NUM_GTH_QUADS*4)-1 downto 0);
  signal sfp_scl_out        : std_logic_vector((c_NUM_GTH_QUADS*4)-1 downto 0);
  signal sfp_sda_oen        : std_logic_vector((c_NUM_GTH_QUADS*4)-1 downto 0);
  signal sfp_scl_oen        : std_logic_vector((c_NUM_GTH_QUADS*4)-1 downto 0);
  signal sfp_detect_in      : std_logic_vector((c_NUM_GTH_QUADS*4)-1 downto 0);
  signal sfp_los_in         : std_logic_vector((c_NUM_GTH_QUADS*4)-1 downto 0);
  signal sfp_tx_fault_in    : std_logic_vector((c_NUM_GTH_QUADS*4)-1 downto 0);
  signal sfp_tx_disable_out : std_logic_vector((c_NUM_GTH_QUADS*4)-1 downto 0);
  signal sfp_led_mode_out   : std_logic_vector((c_NUM_GTH_QUADS*4)-1 downto 0);
  signal sfp_led_synced_out : std_logic_vector((c_NUM_GTH_QUADS*4)-1 downto 0);

  signal sys_rst_n_vec_i    : std_logic_vector(0 downto 0);    --vivado bd generates a reset vector
  signal dac_cs_n_out       : std_logic_vector(1 downto 0);
  
begin


  sys_rst_n_i   <= sys_rst_n_vec_i(0);
  rst_sys       <= not sys_rst_n_i;

  --ref clk from HMC7044
  U_Ref_Clk_Input_Buf : IBUFDS
    generic map 
    (
      DIFF_TERM   => TRUE,
      IOSTANDARD  => "LVDS"
    )
    port map 
    (
      I   => ref_clk_p_i,
      IB  => ref_clk_n_i,
      O   => clk_ref
    );

  --helper clk from Si549
  U_DMTD_Clock_Input_Buf : IBUFDS
    generic map
    (
      DIFF_TERM   => TRUE,
      IOSTANDARD  => "LVDS"
    )
    port map 
    (
      I   => helper_clk_p_i,
      IB  => helper_clk_n_i,
      O =>   clk_dmtd
    );

  U_SWCore_PLL : MMCME3_ADV
    generic map (
      BANDWIDTH            => "OPTIMIZED",
      CLKOUT4_CASCADE      => "FALSE",
      COMPENSATION         => "AUTO",
      STARTUP_WAIT         => "FALSE",
      DIVCLK_DIVIDE        => 1,
      CLKFBOUT_MULT_F      => 15.000,
      CLKFBOUT_PHASE       => 0.000,
      CLKFBOUT_USE_FINE_PS => "FALSE",
      CLKOUT0_DIVIDE_F     => 5.000,
      CLKOUT0_PHASE        => 0.000,
      CLKOUT0_DUTY_CYCLE   => 0.500,
      CLKOUT0_USE_FINE_PS  => "FALSE",
      CLKIN1_PERIOD        => 16.0)
    port map (
      CLKFBOUT => clk_swcore_fbout,
      CLKOUT0  => clk_swcore_bufin,
      CLKFBIN  => clk_swcore_fbin,
      CLKIN1   => clk_sys,
      CLKIN2   => '0',
      CLKINSEL => '1',
      DADDR    => (others => '0'),
      DCLK     => '0',
      DEN      => '0',
      DI       => (others => '0'),
      DWE      => '0',
      CDDCREQ  => '0',
      PSCLK    => '0',
      PSEN     => '0',
      PSINCDEC => '0',
      LOCKED   => swcore_pll_locked,
      PWRDWN   => '0',
      RST      => swcore_pll_rst);

  swcore_pll_rst <= '0'; --rts_gpio_out(0);
  rts_gpio_in(0) <= swcore_pll_locked;

  gpio_in(2 downto 1)     <= fan2_present_i & fan1_present_i;

  U_SWCore_Clock_PLL_Buf : BUFG
    port map (
      I => clk_swcore_bufin,
      O => clk_aux
      );


  U_SWCore_Clock_PLL_FB_Buf : BUFG
    port map (
      I => clk_swcore_fbout,
      O => clk_swcore_fbin
      );

  --zynq processor system top level wrapper
    U_ps_core : entity work.ps_wrapper
    port map
    (
      m_axi_o_araddr   => axi4l_araddr,
      m_axi_o_arprot   => open,
      m_axi_o_arready  => axi4l_arready,
      m_axi_o_arvalid  => axi4l_arvalid,
      m_axi_o_awaddr   => axi4l_awaddr,
      m_axi_o_awprot   => open,
      m_axi_o_awready  => axi4l_awready,
      m_axi_o_awvalid  => axi4l_awvalid,
      m_axi_o_bready   => axi4l_bready,
      m_axi_o_bresp    => axi4l_bresp,
      m_axi_o_bvalid   => axi4l_bvalid,
      m_axi_o_rdata    => axi4l_rdata,
      m_axi_o_rready   => axi4l_rready,
      m_axi_o_rresp    => axi4l_rresp,
      m_axi_o_rvalid   => axi4l_rvalid,
      m_axi_o_wdata    => axi4l_wdata,
      m_axi_o_wready   => axi4l_wready,
      m_axi_o_wstrb    => axi4l_wstrb,
      m_axi_o_wvalid   => axi4l_wvalid,
      axi_clk_i        => clk_sys,
      cpu_irq_n_i      => cpu_irq_n, 
      pl_clk_o         => clk_startup_i,
      pl_rst_n_o       => sys_rst_n_vec_i
    );

  U_axi4lite_bridge : entity work.wb_axi4lite_bridge
    port map (
      clk_sys_i  => clk_sys, 
      rst_n_i    => sys_rst_n_i, 
      ARVALID    => axi4l_arvalid, 
      AWVALID    => axi4l_awvalid, 
      BREADY     => axi4l_bready, 
      RREADY     => axi4l_rready,
      WLAST      => axi4l_wlast,  
      WVALID     => axi4l_wvalid, 
      ARADDR     => axi4l_araddr, 
      AWADDR     => axi4l_awaddr, 
      WDATA      => axi4l_wdata, 
      WSTRB      => axi4l_wstrb, 
      ARREADY    => axi4l_arready, 
      AWREADY    => axi4l_awready, 
      BVALID     => axi4l_bvalid, 
      RLAST      => axi4l_rlast,  
      RVALID     => axi4l_rvalid, 
      WREADY     => axi4l_wready, 
      BRESP      => axi4l_bresp,
      RRESP      => axi4l_rresp,
      RDATA      => axi4l_rdata,
      wb_adr     => host_master_out.adr,
      wb_dat_m2s => host_master_out.dat,
      wb_sel     => host_master_out.sel,
      wb_cyc     => host_master_out.cyc,
      wb_stb     => host_master_out.stb,
      wb_we      => host_master_out.we,
      wb_dat_s2m => host_master_in.dat,
      wb_err     => host_master_in.err,
      wb_rty     => host_master_in.rty,
      wb_ack     => host_master_in.ack,
      wb_stall   => host_master_in.stall
      );


-------------------------------------------------------------------------------
-- GTH IBERT
-------------------------------------------------------------------------------

  gth_qrefclk0    <= mgtclk0_229 & mgtclk0_228 & mgtclk0_227 & mgtclk0_226 & mgtclk0_225 & mgtclk0_224;
  gth_qrefclk00   <= mgtclk0_229 & mgtclk0_228 & mgtclk0_227 & mgtclk0_226 & mgtclk0_225 & mgtclk0_224;
  gth_qrefclk01   <= mgtclk0_229 & mgtclk0_228 & mgtclk0_227 & mgtclk0_226 & mgtclk0_225 & mgtclk0_224;

  gen_ibert: if g_with_ibert = true generate
    U_gth_ibert : ibert_ultrascale_gth_0
      port map
      (
        txn_o             => gth_txn,
        txp_o             => gth_txp,
        rxn_i             => gth_rxn,
        rxp_i             => gth_rxp,
        rxoutclk_o        => open,
        gtrefclk0_i       => gth_qrefclk0,
        gtrefclk1_i       => (others => '0'),
        gtnorthrefclk0_i  => (others => '0'),
        gtnorthrefclk1_i  => (others => '0'),
        gtsouthrefclk0_i  => (others => '0'),
        gtsouthrefclk1_i  => (others => '0'),
        gtrefclk00_i      => gth_qrefclk00,
        gtrefclk10_i      => (others => '0'),
        gtrefclk01_i      => gth_qrefclk01,
        gtrefclk11_i      => (others => '0'),
        gtnorthrefclk00_i => (others => '0'),
        gtnorthrefclk10_i => (others => '0'),
        gtnorthrefclk01_i => (others => '0'),
        gtnorthrefclk11_i => (others => '0'),
        gtsouthrefclk00_i => (others => '0'),
        gtsouthrefclk10_i => (others => '0'),
        gtsouthrefclk01_i => (others => '0'),
        gtsouthrefclk11_i => (others => '0'),
        clk               => mgtclk0_224_gclk
      );
  end generate;

-------------------------------------------------------------------------------
-- GTX PHYs
-------------------------------------------------------------------------------  

  gen_phys_no_ibert: if g_with_ibert = false generate
    gen_phys : for i in 0 to c_NUM_PHYS-1 generate

      -- Instantiate GTX with low phase drift calibration
        U_PHY : entity work.wr_gthe4_phy_family7_xilinx_ip
          generic map (
            g_simulation         => f_bool2int(g_simulation),
            g_use_gclk_as_refclk => false )
          port map (
            clk_gth_i => clk_gth(i),
            clk_freerun_i => clk_sys,

            tx_data_i      => to_phys(i).tx_data,
            tx_k_i         => to_phys(i).tx_k,
            tx_disparity_o => from_phys(i).tx_disparity,
            tx_enc_err_o   => from_phys(i).tx_enc_err,
            tx_out_clk_o => from_phys(i).tx_out_clk,
            rx_rbclk_o     => from_phys(i).rx_clk,
            rx_data_o      => from_phys(i).rx_data,
            rx_k_o         => from_phys(i).rx_k,
            rx_enc_err_o   => from_phys(i).rx_enc_err,
            rx_bitslide_o  => from_phys(i).rx_bitslide,
            rst_i          => to_phys(i).rst,
            loopen_i       => "000",--to_phys(i).loopen,
            pad_txn_o      => gth_txn(i),
            pad_txp_o      => gth_txp(i),
            pad_rxn_i      => gth_rxn(i),
            pad_rxp_i      => gth_rxp(i),
            rdy_o          => from_phys(i).rdy,
            debug_i => x"0000");

      from_phys(i).ref_clk <= clk_ref;

    end generate gen_phys;
  end generate gen_phys_no_ibert;

  gen_terminate_unused_phys : for i in c_NUM_PORTS to c_NUM_PHYS-1 generate
    to_phys(i).tx_data <= (others => '0');
    to_phys(i).tx_k    <= (others => '0');
    to_phys(i).rst     <= '1';
    to_phys(i).loopen  <= '0';
  end generate gen_terminate_unused_phys;


   BUFGMUX_inst : BUFGCTRL
     port map (
       IGNORE0 => '0',
       IGNORE1 => '0',
       CE0 => '1',
       CE1 => '1',
       O   => clk_sys,
       I0  => clk_startup_i,
       I1  => clk_ref,
       S1  => sel_clk_sys,
       S0  => NOT sel_clk_sys); 

  U_Mgtclk0_224_Buf : IBUFDS_GTE4
      generic map (
        REFCLK_EN_TX_PATH  => '0',
        REFCLK_HROW_CK_SEL => "00",
        REFCLK_ICNTL_RX    => "00")
      port map (
        O     => mgtclk0_224,
        ODIV2 => mgtclk0_224_prebuf,
        CEB   => '0',
        I     => gth224_clk0_p_i,
        IB    => gth224_clk0_n_i);

  U_Mgtclk0_225_Buf : IBUFDS_GTE4
      generic map (
        REFCLK_EN_TX_PATH  => '0',
        REFCLK_HROW_CK_SEL => "00",
        REFCLK_ICNTL_RX    => "00")
      port map (
        O     => mgtclk0_225,
        CEB   => '0',
        I     => gth225_clk0_p_i,
        IB    => gth225_clk0_n_i);

  U_Mgtclk0_226_Buf : IBUFDS_GTE4
      generic map (
        REFCLK_EN_TX_PATH  => '0',
        REFCLK_HROW_CK_SEL => "00",
        REFCLK_ICNTL_RX    => "00")
      port map (
        O     => mgtclk0_226,
        CEB   => '0',
        I     => gth226_clk0_p_i,
        IB    => gth226_clk0_n_i);

  U_Mgtclk0_227_Buf : IBUFDS_GTE4
      generic map (
        REFCLK_EN_TX_PATH  => '0',
        REFCLK_HROW_CK_SEL => "00",
        REFCLK_ICNTL_RX    => "00")
      port map (
        O     => mgtclk0_227,
        CEB   => '0',
        I     => gth227_clk0_p_i,
        IB    => gth227_clk0_n_i);

  U_Mgtclk0_228_Buf : IBUFDS_GTE4
      generic map (
        REFCLK_EN_TX_PATH  => '0',
        REFCLK_HROW_CK_SEL => "00",
        REFCLK_ICNTL_RX    => "00")
      port map (
        O     => mgtclk0_228,
        CEB   => '0',
        I     => gth228_clk0_p_i,
        IB    => gth228_clk0_n_i);

  U_Mgtclk0_229_Buf : IBUFDS_GTE4
      generic map (
        REFCLK_EN_TX_PATH  => '0',
        REFCLK_HROW_CK_SEL => "00",
        REFCLK_ICNTL_RX    => "00")
      port map (
        O     => mgtclk0_229,
        CEB   => '0',
        I     => gth229_clk0_p_i,
        IB    => gth229_clk0_n_i);

  U_Mgt_gclk_Buf : BUFG_GT
    port map (
      O       => mgtclk0_224_gclk,
      CE      => '1',
      CEMASK  => '1', 
      CLR     => '0',
      CLRMASK => '1', 
      DIV     => "000",
      I       => mgtclk0_224_prebuf);   
  
  clk_gth(23 downto 20) <= (others => mgtclk0_229);
  clk_gth(19 downto 16) <= (others => mgtclk0_228);
  clk_gth(15 downto 12) <= (others => mgtclk0_227);
  clk_gth(11 downto 8)  <= (others => mgtclk0_226);
  clk_gth(7 downto 4)   <= (others => mgtclk0_225);
  clk_gth(3 downto 0)   <= (others => mgtclk0_224);

  -----------------------------------------------------------------------------
  -- "Bare" top module instantiation
  -----------------------------------------------------------------------------
  U_Real_Top : entity work.scb_top_bare
    generic map (
      g_num_ports       => c_NUM_PORTS,
      g_simulation      => g_simulation,
      g_without_network => false,
      g_with_TRU        => true,
      g_with_TATSU      => true,
      g_with_HWIU       => true,
      g_with_PSTATS     => true,
      g_with_muxed_CS   => false,
      g_with_watchdog   => false,
      g_with_rtu        => true,
      g_with_swc        => true,
      g_with_si57x      => true,
      g_si57x_is_helper => true,
      g_inj_per_EP      => "00" & x"0000",
      g_rtcpu_swap_host_endian => false
    )
    port map 
    (
      sys_rst_n_i         => sys_rst_n_i,
      clk_sys_i           => clk_sys,
      clk_ref_i           => clk_ref,
      clk_dmtd_i          => clk_dmtd,
      clk_aux_i           => clk_aux,
      clk_ext_mul_i       => "00",
      clk_ext_mul_locked_i=> '0',
      clk_aux_p_o         => open,
      clk_aux_n_o         => open,
     -- clk_500_o           => clk_500_o,
      cpu_wb_i            => host_master_out,
      cpu_wb_o            => host_master_in,
      cpu_irq_n_o         => cpu_irq_n,
      pps_i               => '0',
      ppsin_term_o        => open,
      pps_o               => open,
      -- Low-jitter daughterboard support
      ljd_dac_main_sync_n_o => open,
      ljd_dac_main_sclk_o   => open,
      ljd_dac_main_data_o   => open,
      ljd_loopback_i        => '0',
      ljd_loopback_o        => open,
      ljd_clk1_en           => open,
      ljd_clk2_en           => open,
      ljd_detected_o        => open,
      ljd_osc_freq_i        => "000",
      ljd_pll_mosi_o        => open,
      ljd_pll_miso_i        => '0',
      ljd_pll_sck_o         => open,
      ljd_pll_cs_n_o        => open,
      ljd_pll_sync_n_o      => open,
      ljd_pll_reset_n_o     => open,
      ljd_pll_locked_i      => '0',

      dac_helper_load_o     => dac_helper_load,
      dac_helper_value_o    => dac_helper_value,
      dac_main_load_o       => dac_main_load,
      dac_main_value_o      => dac_main_value,
      
      pll_status_i        => '0',
      pll_mosi_o          => hmc_mosi_o,
      pll_miso_i          => hmc_miso_i,
      pll_sck_o           => hmc_sclk_o,
      pll_cs_n_o          => hmc_slen_o,
      pll_sync_n_o        => open,
      pll_reset_n_o       => hmc_reset_n_o,
      uart_txd_o          => uart_txd_o,
      uart_rxd_i          => uart_rxd_i,
      clk_en_o            => open,
      clk_sel_o           => open,
--    uart_sel_o          => uart_sel_o,
      clk_dmtd_divsel_o   => open,
      gpio_i              => gpio_in,
      phys_o              => to_phys(c_NUM_PORTS-1 downto 0),
      phys_i              => from_phys(c_NUM_PORTS-1 downto 0),
--      led_link_o          => led_link_o,
--      led_act_o           => led_act_o(c_NUM_PORTS-1 downto 0),

      --unused, controlled from ps on v4
      i2c_scl_oen_o   => i2c_scl_oen,
      i2c_scl_o       => i2c_scl_out,
      i2c_scl_i       => i2c_scl_in,
      i2c_sda_oen_o   => i2c_sda_oen,
      i2c_sda_o       => i2c_sda_out,
      i2c_sda_i       => i2c_sda_in,

      --sfp i2c
      sfp_i2c_scl_oen_o   => sfp_scl_oen(c_NUM_PORTS-1 downto 0),
      sfp_i2c_scl_o       => sfp_scl_out(c_NUM_PORTS-1 downto 0),
      sfp_i2c_scl_i       => sfp_scl_in(c_NUM_PORTS-1 downto 0),
      sfp_i2c_sda_oen_o   => sfp_sda_oen(c_NUM_PORTS-1 downto 0),
      sfp_i2c_sda_o       => sfp_sda_out(c_NUM_PORTS-1 downto 0),
      sfp_i2c_sda_i       => sfp_sda_in(c_NUM_PORTS-1 downto 0),
      --sfp gpio
      sfp_detect_i        => sfp_detect_in(c_NUM_PORTS-1 downto 0),
      sfp_tx_fault_i      => sfp_tx_fault_in(c_NUM_PORTS-1 downto 0),
      sfp_los_i           => sfp_los_in(c_NUM_PORTS-1 downto 0),
      sfp_tx_disable_o    => sfp_tx_disable_out(c_NUM_PORTS-1 downto 0),
      sfp_led_mode_o      => sfp_led_mode_out(c_NUM_PORTS-1 downto 0),
      sfp_led_synced_o    => sfp_led_synced_out(c_NUM_PORTS-1 downto 0),

      mb_fan1_pwm_o   => open,
      mb_fan2_pwm_o   => open,

      sel_clk_sys_o   => sel_clk_sys,

      rts_si57x_scl_oen_o => rts_si549_scl_oen,
      rts_si57x_sda_oen_o => rts_si549_sda_oen,
      rts_si57x_scl_i => rts_si549_scl_in,
      rts_si57x_sda_i => rts_si549_sda_in,

      aux_scl_i => '1',
      aux_sda_i => '1',
      aux_scl_oen_o => open,
      aux_sda_oen_o => open,
      
      rts_gpio_o => rts_gpio_out,
      rts_gpio_i => rts_gpio_in        
    );

  --gth data tx/rx
  gth224_tx_p_o  <= gth_txp(3 downto 0);
  gth224_tx_n_o  <= gth_txn(3 downto 0);
  gth_rxp(3 downto 0)   <= gth224_rx_p_i; 
  gth_rxn(3 downto 0)   <= gth224_rx_n_i;

  gth225_tx_p_o  <= gth_txp(7 downto 4);
  gth225_tx_n_o  <= gth_txn(7 downto 4);
  gth_rxp(7 downto 4)   <= gth225_rx_p_i;
  gth_rxn(7 downto 4)   <= gth225_rx_n_i;

  gth226_tx_p_o  <= gth_txp(11 downto 8);
  gth226_tx_n_o  <= gth_txn(11 downto 8);
  gth_rxp(11 downto 8)  <= gth226_rx_p_i;
  gth_rxn(11 downto 8)  <= gth226_rx_n_i;  

  gth227_tx_p_o  <= gth_txp(15 downto 12);
  gth227_tx_n_o  <= gth_txn(15 downto 12);
  gth_rxp(15 downto 12)  <= gth227_rx_p_i;
  gth_rxn(15 downto 12)  <= gth227_rx_n_i;    

  gth228_tx_p_o  <= gth_txp(19 downto 16);
  gth228_tx_n_o  <= gth_txn(19 downto 16);
  gth_rxp(19 downto 16)  <= gth228_rx_p_i;
  gth_rxn(19 downto 16)  <= gth228_rx_n_i;   

  gth229_tx_p_o  <= gth_txp(23 downto 20);
  gth229_tx_n_o  <= gth_txn(23 downto 20);
  gth_rxp(23 downto 20)   <= gth229_rx_p_i;
  gth_rxn(23 downto 20)   <= gth229_rx_n_i;

  
  IOBUF_1 : IOBUF
   port map (
     O  => rts_si549_sda_in,
     IO => sda_helper_b,
     I  => '0',
     T  => rts_si549_sda_oen); 

  IOBUF_2 : IOBUF
   port map (
     O  => rts_si549_scl_in,
     IO => scl_helper_b,
     I  => '0',
     T  => rts_si549_scl_oen);

   --use i2c "minibackplane" nets for oled bringup
   IOBUF_OLED_SCL: IOBUF 
    port map (
      O   => i2c_scl_in(0),
      IO  => oled_ctrl_i2c_scl_b,
      I   => i2c_scl_out(0),
      T   => i2c_scl_oen(0));

   IOBUF_OLED_SDA: IOBUF 
    port map (
      O   => i2c_sda_in(0),
      IO  => oled_ctrl_i2c_sda_b,
      I   => i2c_sda_out(0),
      T   => i2c_sda_oen(0));

  ----sfp i2c mapping to gth channel for now...
  ----todo: tidy this up / make module for remapping...
  gen_sfp_scl_in_remap: for i in 0 to (c_NUM_GTH_QUADS/2)-1 generate  
    sfp_scl_in((i*8)+3)   <= sfp_scl_in_array(i*2)(3);      --top rows
    sfp_scl_in((i*8)+2)   <= sfp_scl_in_array(i*2)(2);
    sfp_scl_in((i*8)+1)   <= sfp_scl_in_array(i*2)(1);
    sfp_scl_in(i*8)       <= sfp_scl_in_array(i*2)(0);
    sfp_scl_in((i*8)+7)   <= sfp_scl_in_array((i*2)+1)(3);  --bottom rows
    sfp_scl_in((i*8)+6)   <= sfp_scl_in_array((i*2)+1)(2);
    sfp_scl_in((i*8)+5)   <= sfp_scl_in_array((i*2)+1)(1);
    sfp_scl_in((i*8)+4)   <= sfp_scl_in_array((i*2)+1)(0);
  end generate;

  gen_sfp_sda_in_remap: for i in 0 to (c_NUM_GTH_QUADS/2)-1 generate 
    sfp_sda_in((i*8)+3)   <= sfp_sda_in_array(i*2)(3);
    sfp_sda_in((i*8)+2)   <= sfp_sda_in_array(i*2)(2);
    sfp_sda_in((i*8)+1)   <= sfp_sda_in_array(i*2)(1);
    sfp_sda_in(i*8)       <= sfp_sda_in_array(i*2)(0);
    sfp_sda_in((i*8)+7)   <= sfp_sda_in_array((i*2)+1)(3);
    sfp_sda_in((i*8)+6)   <= sfp_sda_in_array((i*2)+1)(2);
    sfp_sda_in((i*8)+5)   <= sfp_sda_in_array((i*2)+1)(1);
    sfp_sda_in((i*8)+4)   <= sfp_sda_in_array((i*2)+1)(0);
  end generate;  
  
  gen_sfp_scl_out_remap: for i in 0 to (c_NUM_GTH_QUADS/2)-1 generate
    sfp_scl_out_array(i*2)      <= sfp_scl_out((i*8)+3) & sfp_scl_out((i*8)+2) & sfp_scl_out((i*8)+1) & sfp_scl_out(i*8);     --top row of block
    sfp_scl_out_array((i*2)+1)  <= sfp_scl_out((i*8)+7) & sfp_scl_out((i*8)+6) & sfp_scl_out((i*8)+5) & sfp_scl_out((i*8)+4); --bottom row of block
  end generate;

  gen_sfp_sda_out_remap: for i in 0 to (c_NUM_GTH_QUADS/2)-1 generate
    sfp_sda_out_array(i*2)      <= sfp_sda_out((i*8)+3) & sfp_sda_out((i*8)+2) & sfp_sda_out((i*8)+1) & sfp_sda_out(i*8);     --top row of block
    sfp_sda_out_array((i*2)+1)  <= sfp_sda_out((i*8)+7) & sfp_sda_out((i*8)+6) & sfp_sda_out((i*8)+5) & sfp_sda_out((i*8)+4); --bottom row of block
  end generate;

  gen_sfp_scl_oen_remap: for i in 0 to (c_NUM_GTH_QUADS/2)-1 generate
    sfp_scl_oen_array(i*2)      <= sfp_scl_oen((i*8)+3) & sfp_scl_oen((i*8)+2) & sfp_scl_oen((i*8)+1) & sfp_scl_oen(i*8);     --top row of block
    sfp_scl_oen_array((i*2)+1)  <= sfp_scl_oen((i*8)+7) & sfp_scl_oen((i*8)+6) & sfp_scl_oen((i*8)+5) & sfp_scl_oen((i*8)+4); --bottom row of block
  end generate;

  gen_sfp_sda_oen_remap: for i in 0 to (c_NUM_GTH_QUADS/2)-1 generate
    sfp_sda_oen_array(i*2)      <= sfp_sda_oen((i*8)+3) & sfp_sda_oen((i*8)+2) & sfp_sda_oen((i*8)+1) & sfp_sda_oen(i*8);     --top row of block
    sfp_sda_oen_array((i*2)+1)  <= sfp_sda_oen((i*8)+7) & sfp_sda_oen((i*8)+6) & sfp_sda_oen((i*8)+5) & sfp_sda_oen((i*8)+4); --bottom row of block
  end generate;  

  gen_sfp_gpo_remap: for i in 0 to (c_NUM_GTH_QUADS/2)-1 generate 

    lt_sfp_tx_disable_o(i)    <= sfp_tx_disable_out(i*8);
    lct_sfp_tx_disable_o(i)   <= sfp_tx_disable_out((i*8)+1);
    rct_sfp_tx_disable_o(i)   <= sfp_tx_disable_out((i*8)+2);
    rt_sfp_tx_disable_o(i)    <= sfp_tx_disable_out((i*8)+3);
    lb_sfp_tx_disable_o(i)    <= sfp_tx_disable_out((i*8)+4);
    lcb_sfp_tx_disable_o(i)   <= sfp_tx_disable_out((i*8)+5);
    rcb_sfp_tx_disable_o(i)   <= sfp_tx_disable_out((i*8)+6);
    rb_sfp_tx_disable_o(i)    <= sfp_tx_disable_out((i*8)+7);      

    lt_sfp_led_mode_o(i)      <= sfp_led_mode_out(i*8);
    lct_sfp_led_mode_o(i)     <= sfp_led_mode_out((i*8)+1);
    rct_sfp_led_mode_o(i)     <= sfp_led_mode_out((i*8)+2);
    rt_sfp_led_mode_o(i)      <= sfp_led_mode_out((i*8)+3);
    lb_sfp_led_mode_o(i)      <= sfp_led_mode_out((i*8)+4);
    lcb_sfp_led_mode_o(i)     <= sfp_led_mode_out((i*8)+5);
    rcb_sfp_led_mode_o(i)     <= sfp_led_mode_out((i*8)+6);
    rb_sfp_led_mode_o(i)      <= sfp_led_mode_out((i*8)+7);

    lt_sfp_led_synced_o(i)    <= sfp_led_synced_out(i*8);
    lct_sfp_led_synced_o(i)   <= sfp_led_synced_out((i*8)+1);
    rct_sfp_led_synced_o(i)   <= sfp_led_synced_out((i*8)+2);
    rt_sfp_led_synced_o(i)    <= sfp_led_synced_out((i*8)+3);
    lb_sfp_led_synced_o(i)    <= sfp_led_synced_out((i*8)+4);
    lcb_sfp_led_synced_o(i)   <= sfp_led_synced_out((i*8)+5);
    rcb_sfp_led_synced_o(i)   <= sfp_led_synced_out((i*8)+6);
    rb_sfp_led_synced_o(i)    <= sfp_led_synced_out((i*8)+7);

  end generate;

  gen_sfp_gpi_remap: for i in 0 to (c_NUM_GTH_QUADS/2)-1 generate 

    sfp_tx_fault_in(i*8)      <= lt_sfp_tx_fault_i(i);
    sfp_tx_fault_in((i*8)+1)  <= lct_sfp_tx_fault_i(i);
    sfp_tx_fault_in((i*8)+2)  <= rct_sfp_tx_fault_i(i);
    sfp_tx_fault_in((i*8)+3)  <= rt_sfp_tx_fault_i(i);
    sfp_tx_fault_in((i*8)+4)  <= lb_sfp_tx_fault_i(i);
    sfp_tx_fault_in((i*8)+5)  <= lcb_sfp_tx_fault_i(i);
    sfp_tx_fault_in((i*8)+6)  <= rcb_sfp_tx_fault_i(i);
    sfp_tx_fault_in((i*8)+7)  <= rb_sfp_tx_fault_i(i);

    sfp_detect_in(i*8)        <= lt_sfp_detect_i(i);
    sfp_detect_in((i*8)+1)    <= lct_sfp_detect_i(i);
    sfp_detect_in((i*8)+2)    <= rct_sfp_detect_i(i);
    sfp_detect_in((i*8)+3)    <= rt_sfp_detect_i(i);
    sfp_detect_in((i*8)+4)    <= lb_sfp_detect_i(i);
    sfp_detect_in((i*8)+5)    <= lcb_sfp_detect_i(i);
    sfp_detect_in((i*8)+6)    <= rcb_sfp_detect_i(i);
    sfp_detect_in((i*8)+7)    <= rb_sfp_detect_i(i);
  
    sfp_los_in(i*8)           <= lt_sfp_los_i(i);
    sfp_los_in((i*8)+1)       <= lct_sfp_los_i(i);
    sfp_los_in((i*8)+2)       <= rct_sfp_los_i(i);
    sfp_los_in((i*8)+3)       <= rt_sfp_los_i(i);
    sfp_los_in((i*8)+4)       <= lb_sfp_los_i(i);
    sfp_los_in((i*8)+5)       <= lcb_sfp_los_i(i);
    sfp_los_in((i*8)+6)       <= rcb_sfp_los_i(i);
    sfp_los_in((i*8)+7)       <= rb_sfp_los_i(i);

  end generate;

  gen_i2c_buf: for i in 0 to (c_NUM_GTH_QUADS/2)-1 generate
    ------------------------------------------------------------------------------
    --top rows
    ------------------------------------------------------------------------------
    --top left scl
    U_lt_sfp_scl_iobuf: IOBUF 
    port map 
    (
      O   => sfp_scl_in_array(i*2)(0),  
      IO  => lt_sfp_scl_b(i),
      I   => sfp_scl_out_array(i*2)(0),
      T   => sfp_scl_oen_array(i*2)(0)
    );

    --top left sda
    U_lt_sfp_sda_iobuf: IOBUF 
    port map 
    (
      O   => sfp_sda_in_array(i*2)(0),  
      IO  => lt_sfp_sda_b(i),
      I   => sfp_sda_out_array(i*2)(0),
      T   => sfp_sda_oen_array(i*2)(0)
    );

    --top center left scl 
    U_lct_sfp_scl_iobuf: IOBUF 
    port map 
    (
      O   => sfp_scl_in_array(i*2)(1),  
      IO  => lct_sfp_scl_b(i),
      I   => sfp_scl_out_array(i*2)(1),
      T   => sfp_scl_oen_array(i*2)(1)
    );

    --top center left sda
    U_lct_sfp_sda_iobuf: IOBUF 
    port map 
    (
      O   => sfp_sda_in_array(i*2)(1),  
      IO  => lct_sfp_sda_b(i),
      I   => sfp_sda_out_array(i*2)(1),
      T   => sfp_sda_oen_array(i*2)(1)
    );

    --top center right scl
    U_rct_sfp_scl_iobuf: IOBUF 
    port map 
    (
      O   => sfp_scl_in_array(i*2)(2),  
      IO  => rct_sfp_scl_b(i),
      I   => sfp_scl_out_array(i*2)(2),
      T   => sfp_scl_oen_array(i*2)(2)
    );

    --top center right sda
    U_rct_sfp_sda_iobuf: IOBUF 
    port map 
    (
      O   => sfp_sda_in_array(i*2)(2),  
      IO  => rct_sfp_sda_b(i),
      I   => sfp_sda_out_array(i*2)(2),
      T   => sfp_sda_oen_array(i*2)(2)
    );

    --top right scl
    U_rt_sfp_scl_iobuf: IOBUF 
    port map 
    (
      O   => sfp_scl_in_array(i*2)(3),  
      IO  => rt_sfp_scl_b(i),
      I   => sfp_scl_out_array(i*2)(3),
      T   => sfp_scl_oen_array(i*2)(3)
    );    
       
    --top right sda
    U_rt_sfp_sda_iobuf: IOBUF 
    port map 
    (
      O   => sfp_sda_in_array(i*2)(3),  
      IO  => rt_sfp_sda_b(i),
      I   => sfp_sda_out_array(i*2)(3),
      T   => sfp_sda_oen_array(i*2)(3)
    );

    ------------------------------------------------------------------------------
    --bottom rows
    ------------------------------------------------------------------------------
    --bottom left scl
    U_lb_sfp_scl_iobuf: IOBUF 
    port map 
    (
      O   => sfp_scl_in_array((i*2)+1)(0),  
      IO  => lb_sfp_scl_b(i),
      I   => sfp_scl_out_array((i*2)+1)(0),
      T   => sfp_scl_oen_array((i*2)+1)(0)
    );

    --bottom left sda
    U_lb_sfp_sda_iobuf: IOBUF 
    port map 
    (
      O   => sfp_sda_in_array((i*2)+1)(0),  
      IO  => lb_sfp_sda_b(i),
      I   => sfp_sda_out_array((i*2)+1)(0),
      T   => sfp_sda_oen_array((i*2)+1)(0)
    ); 

    --bottom center left scl
    U_lcb_sfp_scl_iobuf: IOBUF 
    port map 
    (
      O   => sfp_scl_in_array((i*2)+1)(1),  
      IO  => lcb_sfp_scl_b(i),
      I   => sfp_scl_out_array((i*2)+1)(1),
      T   => sfp_scl_oen_array((i*2)+1)(1)
    ); 

    --bottom center left sda
    U_lcb_sfp_sda_iobuf: IOBUF 
    port map 
    (
      O   => sfp_sda_in_array((i*2)+1)(1),  
      IO  => lcb_sfp_sda_b(i),
      I   => sfp_sda_out_array((i*2)+1)(1),
      T   => sfp_sda_oen_array((i*2)+1)(1)
    );     

    --bottom center right scl
    U_rcb_sfp_scl_iobuf: IOBUF 
    port map 
    (
      O   => sfp_scl_in_array((i*2)+1)(2),  
      IO  => rcb_sfp_scl_b(i),
      I   => sfp_scl_out_array((i*2)+1)(2),
      T   => sfp_scl_oen_array((i*2)+1)(2)
    ); 

    --bottom center right sda
    U_rcb_sfp_sda_iobuf: IOBUF 
    port map 
    (
      O   => sfp_sda_in_array((i*2)+1)(2),  
      IO  => rcb_sfp_sda_b(i),
      I   => sfp_sda_out_array((i*2)+1)(2),
      T   => sfp_sda_oen_array((i*2)+1)(2)
    ); 

    --bottom right scl
    U_rb_sfp_scl_iobuf: IOBUF 
    port map 
    (
      O   => sfp_scl_in_array((i*2)+1)(3),  
      IO  => rb_sfp_scl_b(i),
      I   => sfp_scl_out_array((i*2)+1)(3),
      T   => sfp_scl_oen_array((i*2)+1)(3)
    ); 

    --bottom right sda
    U_rb_sfp_sda_iobuf: IOBUF 
    port map 
    (
      O   => sfp_sda_in_array((i*2)+1)(3),  
      IO  => rb_sfp_sda_b(i),
      I   => sfp_sda_out_array((i*2)+1)(3),
      T   => sfp_sda_oen_array((i*2)+1)(3)
    );     

  end generate;

  U_WR_DACs : entity work.spec_serial_dac_arb
    generic map(
      g_invert_sclk    => false,
      g_num_extra_bits => 8)
    port map(
      clk_i       => clk_sys,
      rst_n_i     => sys_rst_n_i,
      val1_i      => dac_main_value,
      load1_i     => dac_main_load,
      val2_i      => dac_helper_value,
      load2_i     => dac_helper_load,
      dac_cs_n_o  => dac_cs_n_out,
      dac_clr_n_o => open,
      dac_sclk_o  => dac_sclk_o,
      dac_din_o   => dac_din_o);

    dac_sync_n_o <= dac_cs_n_out(0);  

  --HMC7044 PLL
  IOBUF_HMC_SDATA : IOBUF
  port map 
  (
    O  => hmc_miso_i,
    IO => hmc_sdata_b,
    I  => '0',
    T  => hmc_mosi_o
  );

  hmc_reset_o <= not hmc_reset_n_o;

end Behavioral;


